#include "unification.h"
#include "exception.h"
#include "display.h"
#include "parser.h"
#include "config.h"
#include "automata.h"
#include <math.h>
#include <typeinfo>
// #include <ostream>

bool Automata::isIntersect(Automata *au,int size)
	{
	int *mask1 = this->si->step.front()->bits;
	int *mask2 = au->si->step.front()->bits;
	int i;
	for(i=0;i<size;i++)
		if((mask1[i]!=-1)&&(mask2[i]!=-1))
			return true;
	return false;
	}

State::State(int v, bool s,Automata *a)
	{
	automata = a;
    value = v;
    si = s;
    }

void State::addTransition(Transition* t)
	{
    step.push_back(t);
	}

void State::display(ostream &o)
	{
	if (si)
        o << "INITIAL STATE" << endl;
	else
        o << "State : " << value << endl;
    }

Transition::Transition(State* next, int* b)
	{
    st = next;
    bits = b;
    }

void Transition::display(int bits_size)
	{
        (*mycout) << " BITS : ";
        for (int j=0 ; j < bits_size ; j++) { 
            if (bits[j] == -1 ) {
                (*mycout) << "?";
            } else {
                (*mycout) << bits[j];
            }
        }
        (*mycout) << endl;
    }

/**
 * A constructor that put the equation under the form : 
 *    N = cvalue (from N = a)
 * OR N -aM = cvalue (from N = aM + b)
 * OR N - M - P = 0 = cvalue (from N = M + P)
 * OR N - M = 0 = cvalue (from N = M)
 * and build the associated Automata
 */
Automata::Automata(Condition* n, std::map <int, int>* numvars) {
    // CondA, CondANplusB, CondNplusN, CondN
    if(typeid(*n)==typeid(CondA)) {
#ifdef DEBUG
        (*mycout) << "CondA" << " de type : " << typeid (*n).name() << endl;
#endif
        cvalue = ((CondA*)n)->a;
        a.push_back(1);
		nvars = 1;
    } else if(typeid(*n)==typeid(CondANplusB)) {
#ifdef DEBUG
        (*mycout) << "CondANplusB" << " de type : " << typeid (*n).name() << endl;
#endif
        cvalue = ((CondANplusB*)n)->b;
		nvars = 2;
        a.push_back(1);
        a.push_back(- ((CondANplusB*)n)->a );
    } else if(typeid(*n)==typeid(CondNplusN)) {
#ifdef DEBUG
        (*mycout) << "CondNplusN" << " de type : " << typeid (*n).name() << endl;
#endif
        cvalue = 0;
		nvars = 3;
        a.push_back(1);
        a.push_back(-1);
        a.push_back(-1);
    } else { // CondN
#ifdef DEBUG
        (*mycout) << "CondN" << " de type : " << typeid (*n).name() << endl;
#endif
        cvalue = 0;
		nvars = 2;
        a.push_back(1);
        a.push_back(-1);
    }

    /** We're now in the middle of the constructor : 
     * in a[] the coefficients of the NumVars, 
     * in cvalue the result (right part of the equation), 
     * in numvars[] the map between a NumericalVar* and its integer indice,
     * in numvars->size() the size of the bits[] table of the Transitions.
     */
    
    si = new State(INT_MIN, true,this);
    State* final_st = new State(cvalue, false,this);
    vector<State*> table_of_st;
    table_of_st.push_back(final_st);
    list<State*> active_st;
    active_st.push_back(final_st);
    int bits_size = numvars->size();

    while(!active_st.empty()) {
        State* tmpst = active_st.front();
        active_st.pop_front();
#ifdef DEBUG
        (*mycout) << "I just poped : ";
        tmpst->display(*mycout);
#endif
        for (int i = 0 ; i < (pow(2.f, nvars)) ; i++){
            int decal = i;
            int* bits;
            bits = new int[bits_size];
			int *prebits = new int[nvars];
            for (int j=0 ; j < nvars ; j++) {
                prebits[j] = (0x0001 & decal);
                // (*mycout) << "PREBITS : " << prebits[j] << " DE " << j << endl;
                decal = decal >> 1;
                // (*mycout) << "**PREBITS : " << prebits[j] << " DE " << j << endl;

            }

            for (int k=0 ; k < bits_size ; k++) bits[k] = -1;

            if(typeid(*n)==typeid(CondA)) {
                bits[((*numvars)[n->n1->id])] = prebits[0];
            } else if(typeid(*n)==typeid(CondANplusB)) {
                bits[((*numvars)[n->n1->id])] = prebits[0];
                bits[((*numvars)[((CondANplusB*)n)->n2->id])] = prebits[1];
            } else if(typeid(*n)==typeid(CondNplusN)) {
                bits[((*numvars)[n->n1->id])] = prebits[0];
                bits[((*numvars)[((CondNplusN*)n)->n2->id])] = prebits[1];
                bits[((*numvars)[((CondNplusN*)n)->n3->id])] = prebits[2];
            } else { // CondN
                bits[((*numvars)[n->n1->id])] = prebits[0];
                bits[((*numvars)[((CondN*)n)->n2->id])] = prebits[1];
            }
            
#ifdef DEBUG
			/*
            (*mycout) << "* BITS : ";
            for (int j=0 ; j < bits_size ; j++) { 
                if (bits[j] == -1 ) {
                    (*mycout) << "?";
                } else {
                (*mycout) << bits[j];
                }
            }
            (*mycout) << endl;
            (*mycout) << "* PREBITS : ";
            for (int j=0 ; j < nvars ; j++) (*mycout) << prebits[j];
            (*mycout) << endl;
			*/
#endif
            // calculus of -a*bits
            int ab;
            if(typeid(*n)==typeid(CondA)) {
                ab = - (a[0]*bits[(*numvars)[n->n1->id]]);
            } else if(typeid(*n)==typeid(CondANplusB)) {
                ab = - (a[0]*bits[(*numvars)[n->n1->id]] + 
                              a[1]*bits[(*numvars)[((CondANplusB*)n)->n2->id]]);
            } else if(typeid(*n)==typeid(CondNplusN)) {
                ab = -(a[0]*bits[(*numvars)[n->n1->id]] +
                       a[1]*bits[(*numvars)[((CondNplusN*)n)->n2->id]] + 
                       a[2]*bits[(*numvars)[((CondNplusN*)n)->n3->id]]);
            } else { // CondN
                ab = -(a[0]*bits[(*numvars)[n->n1->id]] +
                       a[1]*bits[(*numvars)[((CondN*)n)->n2->id]]);
            }
#ifdef DEBUG
            /// (*mycout) << "Value of a*b : " << ab << endl;
            /// (*mycout) << "Value of the current state : " << tmpst->value << endl;
#endif

            if (ab == tmpst->value) {
                Transition* tr = new Transition(tmpst,bits);
                si->addTransition(tr);
				#ifdef DEBUG
				(*mycout) << "Link to initial" << endl;
				(*mycout) << " si -> " << tmpst->value << endl;
				tr->display(bits_size);
				#endif
            } //else {
            // calculus of gamma 0 (g0)
            double g0;
            if(typeid(*n)==typeid(CondA)) {
                g0 = ((double)(tmpst->value - (a[0]*bits[(*numvars)[n->n1->id]])) ) / 2;
            } else if(typeid(*n)==typeid(CondANplusB)) {
                g0 = ((double)(tmpst->value - (a[0]*bits[(*numvars)[n->n1->id]] + 
                            a[1]*bits[(*numvars)[((CondANplusB*)n)->n2->id]])) ) / 2;
            } else if(typeid(*n)==typeid(CondNplusN)) {
                g0 = (double)(tmpst->value - (a[0]*bits[(*numvars)[n->n1->id]] +
                        a[1]*bits[(*numvars)[((CondNplusN*)n)->n2->id]] + 
                        a[2]*bits[(*numvars)[((CondNplusN*)n)->n3->id]])) / 2;
            } else { // CondN
                g0 = (double)(tmpst->value - (a[0]*bits[(*numvars)[n->n1->id]] +
                        a[1]*bits[(*numvars)[((CondN*)n)->n2->id]])) / 2;
            }
#ifdef DEBUG
            /// (*mycout) << "Value of gamma0 : " << g0 << endl;
#endif
            // if (floor(g0) == floor(g0 + 0.6)) {
            double double_g0;
            if (!modf(g0, &double_g0)) {
                int int_g0 = int(double_g0);
                bool found = false;
                // search for a state with the value g0
                vector<State*>::iterator my_it = table_of_st.begin(); // MIAM
                while (my_it != table_of_st.end()) {
                    if ((*my_it)->value == int_g0) { 
                        found = true;
                        Transition* tr = new Transition(tmpst,bits);
                        (*my_it)->addTransition(tr);
						#ifdef DEBUG
						(*mycout) << " " << (*my_it)->value << " -> " << tmpst->value << endl;
						tr->display(bits_size);
						#endif
                    }
                    my_it++;
                }

                // if not found, add it
                if (!found) {
                    State* newst = new State(int_g0,false,this);
                    table_of_st.push_back(newst);
                    active_st.push_back(newst);
                    Transition* tr = new Transition(tmpst,bits);
                    newst->addTransition(tr);
					#ifdef DEBUG
					(*mycout) << " " << newst->value << " -> " << tmpst->value << endl;
					tr->display(bits_size);
					#endif
                }
            }
            //}
        }
    }
}

Automata::Automata(Condition* n) {
}

Automata::~Automata(){
}

void Automata::display(ostream &o) {
    o << "Automata of value : " << cvalue << " and with a : "; 
    si->display(*mycout);
}


