#include "config.h"
#include "exception.h"

#include <string.h>
#include <fstream>
#include <iostream>

#include <typeinfo>

ostream *mycout;
Configuration *globconfig;

/**
* Print options help message
*/
void Configuration::printOptionInfo()
	{
	(*mycout) << endl;
	(*mycout) << " Unificator Options" << endl;
	(*mycout) << endl;
	(*mycout) << "unificator [OPTIONS] [FILE]" << endl;
	(*mycout) << endl;
	(*mycout) << "Options:" << endl;
	(*mycout) << " -help,-h     : print this message" << endl;
	(*mycout) << " -step,-s     : pause on every step of the unification" << endl;
	(*mycout) << " -beep,-b     : emit a beep on every step of the unification and at the end (need \"beep\" for linux)" << endl;
	(*mycout) << " -nopbv       : do not apply Presburger automata verification" << endl;
	(*mycout) << " -anniversary : emit a special beep on every step of the unification and at the end" << endl;
	(*mycout) << " -info,-i     : print unification applied rules" << endl;
	//(*mycout) << " -zero,-z     : allow numerical variable to be nil" << endl;
    (*mycout) << " -latex,-l    : print the steps and the final result in latex" << endl;
    (*mycout) << " -output,-o   : redirect standard output in a file" << endl;
	(*mycout) << endl;
	(*mycout) << "If [FILE] is not given, the standard input is used to get data." << endl; 
	printConsol();
	}

/**
* Print command help message
*/
void Configuration::printConsol()
	{
	(*mycout) << endl;
	(*mycout) << " Unificator Syntax" << endl;
	(*mycout) << endl;
	(*mycout) << "Commands:" << endl;
	(*mycout) << " [TERM]       : new term (see documentation for more details)" << endl;
	(*mycout) << " help         : print this message" << endl;
	(*mycout) << " set [OPTION] : activate the option" << endl;
	(*mycout) << " set [OPTION] : activate the option" << endl;
	(*mycout) << endl;
	(*mycout) << "Options:" << endl;
	(*mycout) << " step : pause on every step of the unification" << endl;
	(*mycout) << " beep : emit a beep on every step of the unification" << endl;
	(*mycout) << " info : print unification applied rules" << endl;
	//(*mycout) << " zero : allow numerical variable to be nil" << endl;
	(*mycout) << endl;
	}

/**
* Parse configuration from options
*/
Configuration::Configuration(int argc, char *argv[])
	{
	inputdatas = NULL;
	mycout = NULL;
	printRule = false;
	pauseEveryStep = false;
	NcanbeZero = false;
	fileInput = false;
	presBurgerVerif = true;
	disjonction = true;
	writeMode = Configuration::NORMAL;
	beep = 0;
	bool askhelp = false;

	for(int i=1;i<argc;i++)
		{
		if(argv[i][0]=='-')
			{
			if((!strcmp(argv[i],"-step"))||(!strcmp(argv[i],"-s")))
				pauseEveryStep = true;
			else if((!strcmp(argv[i],"-info"))||(!strcmp(argv[i],"-i")))
				printRule = true;
			/*else if((!strcmp(argv[i],"-zero"))||(!strcmp(argv[i],"-z")))
				NcanbeZero = true;*/
			else if((!strcmp(argv[i],"-beep"))||(!strcmp(argv[i],"-b")))
				beep = 1;
			else if((!strcmp(argv[i],"-invbeep"))||(!strcmp(argv[i],"-ib")))
				beep = 2;
			else if((!strcmp(argv[i],"-anniversary"))||(!strcmp(argv[i],"-ann")))
				beep = 3;
			else if((!strcmp(argv[i],"-latex"))||(!strcmp(argv[i],"-l")))
				writeMode = Configuration::LATEX;
			else if(!strcmp(argv[i],"-nopbv"))
				presBurgerVerif = false;
			else if(!strcmp(argv[i],"-nodisj"))
				disjonction = false;
			else if((!strcmp(argv[i],"-output"))||(!strcmp(argv[i],"-o")))
				{
				if(!mycout)
					{
					i++;
					if(i>=argc)
						throw OptionException("no output file given");
					mycout = new fstream(argv[i],ios::out);
					if(mycout->fail())
						throw OptionException("can't open the output file");
					}
				else
					throw OptionException("several ouput files");
				}
			else if((!strcmp(argv[i],"-help"))||(!strcmp(argv[i],"-h")))
				askhelp = true;
			else
				{
				throw OptionException(string("unknow option \"") + argv[i] + "\"");
				}
			}
		else
			{
			if(!inputdatas)
				{
				fileInput = true;
				inputdatas = new fstream(argv[i],ios::in);
				if(inputdatas->fail())
					throw OptionException("can't open the input file");
				}
			else
				throw OptionException("several input files");
			}
		}
	if(!mycout)
		mycout = &cout;		
	if(!inputdatas)
		inputdatas = &cin;

	if(askhelp)
		{
		printOptionInfo();
		exit(0);
		}
	}
	
Configuration::~Configuration()
	{
	
	}

