#include "display.h"
#include "exception.h"
#include <iostream>

#ifdef WIN32
#include <windows.h>
#else

#endif

namespace Display
	{

/**
* Print a new line in latex syntax
* @param s output stream
*/
void texEndl(std::ostream& s)
	{
	if(globconfig->writeMode==Configuration::LATEX)
		s << "\\\\" << endl;
	}

/**
* Print a # in latex syntax
* @param s output stream
*/
void texDi(std::ostream& s)
	{
	if(globconfig->writeMode==Configuration::NORMAL)
		s << "#";
	}

/**
* Print a 20 pt space in latex syntax
* @param s output stream
*/
void texHspace(std::ostream& s)
	{
	if(globconfig->writeMode==Configuration::LATEX)
		s << "\\hspace{20pt}";
	}

/**
* Print the head of the latex
*/
void Head(Configuration *config)
	{
	if(config->writeMode==Configuration::LATEX)
		{
		(*mycout) << "\\documentclass[a4paper,10pt]{article}" << endl;
		(*mycout) << "\\newcommand{\\noi}{\\noindent}" << endl;
		(*mycout) << "\\newcommand{\\II}{\\mbox{\\large 1\\hskip -0,353em 1}}" << endl;
		(*mycout) << "\\newcommand{\\MR}{\\mathcal{R}}" << endl;
		(*mycout) << "\\newcommand{\\RR} {\\mbox{I \\hskip -0.55 em R}}" << endl;
		(*mycout) << "\\usepackage[utf8]{inputenc}" << endl;
		(*mycout) << "\\usepackage{amsmath,amssymb}" << endl;
		(*mycout) << "\\usepackage{fullpage}" << endl;
		(*mycout) << "\\usepackage{graphicx}" << endl;
		(*mycout) << "\\begin{document}" << endl;
		}
	}

/**
* Print the foot of the latex
*/
void Foot(Configuration *config)
	{
	if(config->writeMode==Configuration::LATEX)
		{
		(*mycout) << "\\end{document}" << endl;
		}
	}
	
/**
* Print the result of the unification (and play sound if needed)
*/
void displayResult(Unification *unification)
	{
	if(unification->result)
		{
		Display::texEndl(*mycout);
		(*mycout) << endl;
		Display::texDi(*mycout);
		(*mycout) << "Unification Succeeds";
		Display::texDi(*mycout);
		Display::texEndl(*mycout);
		(*mycout) << endl << endl;

		if(unification->configuration->disjonction)
			{
			(*mycout) << "Number of main solution : " << unification->solutions.size();
			Display::texEndl(*mycout);
			(*mycout) << endl << endl;
			list<Solution*>::iterator it = unification->solutions.begin();
			int num = 0;
			while(it!=unification->solutions.end())
				{
				num++;
				(*mycout) << "solution " << num ;
				Display::texEndl(*mycout);
				(*mycout) << endl;
				(*it)->display(*mycout);
				if(unification->configuration->writeMode==Configuration::LATEX)
					(*mycout) << "\\null";
				Display::texEndl(*mycout);
				(*mycout) << endl;
				it++;
				}
			}

		if(unification->configuration->beep!=0)
			{
			if(unification->configuration->beep==3)
				{
				beep(1046,250);beep(1046,250);
				beep(1175,500);beep(1046,500);
				beep(1397,500);beep(1318,1000);
				beep(1046,250);beep(1046,250);
				beep(1175,500);beep(1046,500);
				beep(1568,500);beep(1397,1000);
				beep(1046,250);beep(1046,250);
				beep(2093,500);beep(1760,500);
				beep(1397,500);beep(1318,500);
				beep(1175,1000);beep(1868,250);
				beep(1868,250);beep(1760,500);
				beep(1397,500);beep(1568,500);
				beep(1397,1500);
				}
			else
				{
				beep(500,100);
				beep(600,100);
				}
			}

		}
	else
		{
		(*mycout) << endl;
		Display::texDi(*mycout);
		(*mycout) << "Unification Failed";
		Display::texDi(*mycout);
		Display::texEndl(*mycout);
		(*mycout) << endl;
		if(unification->configuration->beep!=0)
			{
			beep(50,100);
			beep(40,100);
			beep(50,100);
			}
		}
	}

/**
* Print the duration
*/
void displayTime(long atime)
	{

	if(atime>1000*60*60*24)
		{
		(*mycout) << (atime / (1000 * 60*60*24));
		(*mycout) << " day(s) ";
		}

	if(atime>1000*60*60)
		{
		(*mycout) << (atime / (1000 * 60*60));
		(*mycout) << " hour(s) ";
		}

	if(atime>1000*60)
		{
		(*mycout) << (atime / (1000 * 60));
		(*mycout) << " minute(s) ";
		}

	(*mycout) << (((float)(atime%(1000*60)))/1000);
	(*mycout) << " seconde(s) ";

	}
	
	void beep(int frec,int time)
		{
#ifdef WIN32
		Beep(frec,time);
#else
		char tmp[100];
		sprintf(tmp,"beep -f %i -l %i",frec,time);
		//sprintf(tmp,"xset b 100 %i %i && echo -e \"\\a\" &",frec,time);
		//printf("%s\n",tmp);
		system(tmp);
		
		//printf("\a");
#endif
		}

	}

/**
* Print a list of uint
*/
std::ostream& operator << (std::ostream& o, const list<unsigned int> &path)
	{
	list<unsigned int>::const_iterator it = path.begin();
	while(it!=path.end())
		{
		o << (*it) << " ";
		it++;
		}
	return o;
	}
	


