#include "FlexLexer.h"
#include "parser.h"
#include "exception.h"
#include <typeinfo>
#include <sstream>

// bon with the parser
extern int yylex();
extern void set_extParser(Parser* extP);
extern void set_result(Node **r);
extern yyFlexLexer* lexer;
extern int numLine;
extern int yyparse (void);

int Node::ndterms;

/**
 * Parse input data and check soundness
 */
Parser::Parser(Configuration *config)
	{
	this->config = config;
	autoNumericalValCur = 0;
	autoContstantValCur = 0;

	set_extParser(this);
	this->result = NULL;
	set_result(&this->result);

	lexer = new yyFlexLexer(config->inputdatas,mycout);

	numLine = 1;
	if((yyparse())||(!this->result))
		exit(1);	
	
	//============test 1==========
	// f(g(a,y),y) = f(x,b)

	/*
	this->result = new And();
	Function *f1 = new Function("f",this);
	Function *f2 = new Function("f",this);
	Function *f3 = new Function("f",this);

	f2->addArgument(new Function("a",this));
	f2->addArgument(new Variable("y",this));

	f1->addArgument(f2);
	f1->addArgument(new Variable("y",this));

	f3->addArgument(new Variable("x",this));
	f3->addArgument(new Function("b",this));
	((And*)(result))->addBranch(new Equal(f1,f3));
	((And*)(result))->addBranch(new Equal(new Variable("z",this),new Variable("z",this))); 
    */

	//============test 2==========
	// f(@,a)^{N}.x = f(y,f(a,@)^{M}.z)

	/*
	this->result = new And();

	Function *f2 = new Function("f",this);
	f2->addArgument(new Hole());
	f2->addArgument(new Function("a",this));

	Function *f3 = new Function("f",this);
	f3->addArgument(new Function("a",this));
	f3->addArgument(new Hole());

	Function *f1 = new Function("f",this);
	f1->addArgument(new Variable("y",this));
	f1->addArgument(new Expo(f3,new NumericalVar("M",this),new Variable("z",this)));

	((And*)(result))->addBranch(new Equal(new Expo(f2,new NumericalVar("N",this),new Variable("x",this)),f1));
	*/
	
	//============test 3==========
	// f(@,a)^{2}.x = f(y,f(a,@)^{2}.z)
	/*
	this->result = new And();

	Function *f2 = new Function("f",this);
	f2->addArgument(new Hole());
	f2->addArgument(new Function("a",this));

	Function *f3 = new Function("f",this);
	f3->addArgument(new Function("a",this));
	f3->addArgument(new Hole());

	Function *f1 = new Function("f",this);
	f1->addArgument(new Variable("y",this));
	f1->addArgument(new Expo(f3,2,new Variable("z",this)));

	((And*)(result))->addBranch(new Equal(new Expo(f2,2,new Variable("x",this)),f1));
	*/

	//============test 4==========
	// f(@,a)^N.y = x
/*
	this->result = new And();

	Function *f2 = new Function("f",this);
	f2->addArgument(new Hole());
	f2->addArgument(new Function("a",this));

	((And*)(result))->addBranch(new Equal(new Expo(f2,new NumericalVar("N",this),new Variable("y",this)),new Variable("x",this)));
	*/

	//============test 5==========
	// g(f(a,g(@))^{N1}.z) = g(f(x,g(@)))^{N2}.y
	/*
	this->result = new And();

	Function *f6 = new Function("g",this);
	f6->addArgument(new Hole());

	Function *f5 = new Function("f",this);
	f5->addArgument(new Variable("x",this));
	f5->addArgument(f6);

	Function *f4 = new Function("g",this);
	f4->addArgument(f5);

	Function *f3 = new Function("g",this);
	f3->addArgument(new Hole());

	Function *f2 = new Function("f",this);
	f2->addArgument(new Function("a",this));
	f2->addArgument(f3);

	Function *f1 = new Function("g",this);
	f1->addArgument(new Expo(f2,new NumericalVar("N1",this),new Variable("z",this)));

	((And*)(result))->addBranch(new Equal(f1,new Expo(f4,new NumericalVar("N2",this),new Variable("y",this))));
	*/
	//==========================
	
	checkSoundness();
	}
	
/**
 * generation of an intermediate name for numerical variable
 */
string Parser::genNumericalValName()
	{
	ostringstream text;
	if(this->config->writeMode==Configuration::LATEX)
		text << "\\_N";
	else
		text << "_N";
	text << autoNumericalValCur;
	autoNumericalValCur++;
	return text.str();
	}

/**
 * generation of an intermediate name for variable
 */
string Parser::genContstantValName()
	{
	ostringstream text;
	if(this->config->writeMode==Configuration::LATEX)
		text << "\\_C";
	else	
		text << "_C";
	text << autoContstantValCur;
	autoContstantValCur++;
	return text.str();
	}
	
void Parser::checkSoundness()
	{
	Node::ndterms = 0;
	result->checkSoundnessAndCalcul();
	}
	
//==============================================================

void Node::print(std::ostream& o)
	{
	o << "<?>";
	}
	
void Condition::print(std::ostream& o)
	{
	o << "<?>";
	}

void And::printNoBracket(std::ostream& o)
	{
	list<Condition*>::iterator it = conditions.begin();
	while(it!=conditions.end())
		{
		(*it)->print(o);
		if((*it!=conditions.back())||(!branchs.empty()))
		if(globconfig->writeMode==Configuration::LATEX)
			o << "\\wedge ";
		else
			o << " and ";
		it++;
		}

	list<Node*>::iterator it2 = branchs.begin();
	while(it2!=branchs.end())
		{
		(*it2)->print(o);
		if(*it2!=branchs.back())
		if(globconfig->writeMode==Configuration::LATEX)
			o << "\\wedge ";
		else
			o << " and ";
		it2++;
		}
	}

void And::print(std::ostream& o)
	{
	o << " (";
	printNoBracket(o);
	o << ") ";
	}
	
void Equal::print(std::ostream& o)
	{
	n1->print(o);
	o << " = ";
	n2->print(o);
	}
	
void Variable::print(std::ostream& o)
	{
	o << parser->variabletab[id].name ;
	}

void Function::print(std::ostream& o)
	{
	o << parser->fonctiontab[id].name ;
	if(!arguments.empty())
		{
		o << "(";
		vector<Node*>::iterator it = arguments.begin();
		while(it!=arguments.end())
			{
			(*it)->print(o);
			if((*it)!=arguments.back())
				o << ",";
			it++;
			}
		o << ")";
		}
	}
	
void NumericalVar::print(std::ostream& o)
	{
	o << parser->numericalvariabletab[id].name ;
	}

void CondA::print(std::ostream& o)
	{
	n1->print(o);
	o << "=" << a ;
	}

void CondANplusB::print(std::ostream& o)
	{
	n1->print(o);
	o << "=" ;
	if(a!=1)
		o << a << "*";
	n2->print(o);
	if(b!=0)
	o << "+" << b;
	}
	
void CondNplusN::print(std::ostream& o)
	{
	n1->print(o);
	o << "=";
	n2->print(o);
	o << "+";
	n3->print(o);
	}
	
void CondN::print(std::ostream& o)
	{
	n1->print(o);
	o << "=";
	n2->print(o);
	}
	
void Expo::print(std::ostream& o)
	{
	if(typeid(*n1)==typeid(Expo))
		o << "(";
	n1->print(o);
	if(typeid(*n1)==typeid(Expo))
		o << ")";
	o << "^{";
	if(free)
		var->print(o);
	else
		o << expNum;
	o << "}.";
	if(typeid(*n2)==typeid(Expo))
		o << "(";
	n2->print(o);
	if(typeid(*n2)==typeid(Expo))
		o << ")";
	}
	
void Or::print(std::ostream& o)
	{
	o << " (" <<endl;
	list<Node*>::iterator it = branchs.begin();
	while(it!=branchs.end())
		{
		(*it)->print(o);
		if(*it!=branchs.back())
		if(globconfig->writeMode==Configuration::LATEX)
			o << "\\vee ";
		else
			o << " or " << endl;
		it++;
		}
	o << endl <<" ) ";
	}

//===============================================================

Node::Node()
	{
	rhoterm = false;
	ndHole = false;
	holePosition = NULL;
	}

Node::~Node()
	{
	}


Hole::~Hole()
	{
	if(holePosition)
		delete holePosition;
	}

Hole::Hole()
	: Node()
	{
	holePosition = new list<unsigned int>();
	}
	
Condition::~Condition()
	{
	}
	
NumericalVar::NumericalVar(const char* name,Parser *parser,bool automatic)
	{
	this->parser = parser;
	vector<NumericalVariableTabElement>::iterator it = parser->numericalvariabletab.begin();
	int num=0;
	while(it!=parser->numericalvariabletab.end())
		{
		if(it->name==name)
			{
			this->id = num;
			if(automatic!=(*it).automatic)
				throw SytaxException("using of reserved name for numerical variable");
			return;
			}
		it++;
		num++;
		}
	NumericalVariableTabElement add;
	add.name = name;
	add.automatic = automatic;
	parser->numericalvariabletab.push_back(add);
	this->id = num;
	}

Function::Function(int id,Parser *parser)
	: Node()
	{
	this->id = id;
	this->parser = parser;
	}

Function::Function()
	: Node()
	{
	this->id = -1;
	this->parser = NULL;
	}

void Function::setName(const char* name,Parser *parser)
	{
	this->parser = parser;
	vector<FonctionTabElement>::iterator it = parser->fonctiontab.begin();
	int num=0;
	while(it!=parser->fonctiontab.end())
		{
		if(it->name==name)
			{
			this->id = num;
			return;
			}
		it++;
		num++;
		}
	FonctionTabElement add;
	add.name = name;
	add.arity = -1;
	parser->fonctiontab.push_back(add);
	this->id = num;
	}
	
Or::Or()
	: Node()
	{
	
	}
	
void Or::addBranch(Node *n)
	{
	branchs.push_back(n);
	}

CondA::CondA(NumericalVar* n1,int a)
	{
	this->n1 = n1;
	this->a = a;
	}

CondANplusB::CondANplusB(NumericalVar* n1,int a,NumericalVar* n2,int b)
	{
	this->n1 = n1;
	this->n2 = n2;
	this->a = a;
	this->b = b;
	}
	
CondNplusN::CondNplusN(NumericalVar* n1,NumericalVar* n2,NumericalVar* n3)
	{
	this->n1 = n1;
	this->n2 = n2;
	this->n3 = n3;	
	}	

CondN::CondN(NumericalVar* n1,NumericalVar* n2)
	{
	this->n1 = n1;
	this->n2 = n2;
	}
	
And::And()
	: Node()
	{
	
	}
	
void And::addBranch(Node *n)
	{
	branchs.push_back(n);
	}
	
void And::addCondition(Condition *c)
	{
	conditions.push_back(c);
	}
	
Equal::Equal(Node *n1,Node *n2)
	: Node()
	{
	this->n1 = n1;
	this->n2 = n2;
	}

Variable::Variable(const char *name,Parser *parser)
	: Node()
	{
	this->parser = parser;
	vector<VariableTabElement>::iterator it = parser->variabletab.begin();
	int num=0;
	while(it!=parser->variabletab.end())
		{
		if(it->name==name)
			{
			this->id = num;
			return;
			}
		it++;
		num++;
		}
	VariableTabElement add;
	add.name = name;
	parser->variabletab.push_back(add);
	this->id = num;
	}
	
Function::Function(const char* name,Parser *parser)
	: Node()
	{
	this->parser = parser;
	vector<FonctionTabElement>::iterator it = parser->fonctiontab.begin();
	int num=0;
	while(it!=parser->fonctiontab.end())
		{
		if(it->name==name)
			{
			this->id = num;
			return;
			}
		it++;
		num++;
		}
	FonctionTabElement add;
	add.name = name;
	add.arity = -1;
	parser->fonctiontab.push_back(add);
	this->id = num;
	}
	
void Function::addArgument(Node *n)
	{
	arguments.push_back(n);
	}

Expo::Expo(Node *n1,int num,Node *n2)
	: Node()
	{
	this->n1 = n1;
	this->n2 = n2;
	this->expNum = num;
	free = false;
	}

Expo::Expo(Node *n1,NumericalVar *num,Node *n2)
	: Node()
	{
	this->n1 = n1;
	this->n2 = n2;
	this->var = num;
	free = true;	
	}

void Hole::print(std::ostream& o)
	{
	if(globconfig->writeMode==Configuration::LATEX)
		o << "\\diamond ";
	else
		o << "@";
	}

void Clash::print(std::ostream& o)
	{
	if(globconfig->writeMode==Configuration::LATEX)
		o << "\\bot ";
	else
		o << "<Clash>";
	}

void Trivial::print(std::ostream& o)
	{
	if(globconfig->writeMode==Configuration::LATEX)
		o << "\\top ";
	else
		o << "<Triv>";
	}

void Node::checkSoundnessAndCalcul()
	{
	Node::ndterms++;
	this->ndHole = 0;
	this->rhoterm = false;
	}

void Hole::checkSoundnessAndCalcul()
	{
	Node::ndterms++;
	this->ndHole = 1;
	this->rhoterm = false;
	this->holePosition->clear();
	}

void And::checkSoundnessAndCalcul()
	{
	Node::ndterms++;
	list<Node*>::iterator it = branchs.begin();
	rhoterm = false;
	ndHole = 0;
	while(it!=branchs.end())
		{
		(*it)->checkSoundnessAndCalcul();
		if((*it)->rhoterm)
			rhoterm = true;
		ndHole += (*it)->ndHole;
		it++;
		}
	}

void Or::checkSoundnessAndCalcul()
	{
	Node::ndterms++;
	list<Node*>::iterator it = branchs.begin();
	rhoterm = false;
	ndHole = 0;
	while(it!=branchs.end())
		{
		(*it)->checkSoundnessAndCalcul();
		if((*it)->rhoterm)
			rhoterm = true;
		ndHole += (*it)->ndHole;
		it++;
		}
	}

void Function::checkSoundnessAndCalcul()
	{
	Node::ndterms++;
	rhoterm = false;
	ndHole = 0;
	if(parser->fonctiontab[id].arity==-1)
		parser->fonctiontab[id].arity = (int)arguments.size();
	else
		if(arguments.size()!=(unsigned int)parser->fonctiontab[id].arity)
			throw SytaxException("fonction \"" + parser->fonctiontab[id].name + "\" with several arities");

	vector<Node*>::iterator it = arguments.begin();
	int numArg = 0;
	while(it!=arguments.end())
		{
		(*it)->checkSoundnessAndCalcul();
		if((*it)->rhoterm)
			rhoterm = true;
		ndHole += (*it)->ndHole;
		if((*it)->ndHole>0)
			{
			this->holePosition = (*it)->holePosition;
			this->holePosition->push_front(numArg);
			}
		it++;
		numArg++;
		}
	}

void Variable::checkSoundnessAndCalcul()
	{
	Node::ndterms++;
	rhoterm = false;
	ndHole = 0;
	}

void Equal::checkSoundnessAndCalcul()
	{
	Node::ndterms++;
	rhoterm = false;
	ndHole = 0;
	n1->checkSoundnessAndCalcul();
	n2->checkSoundnessAndCalcul();

	if(n1->rhoterm)
		rhoterm = true;
	ndHole += n1->ndHole;
	
	if(n2->rhoterm)
		rhoterm = true;
	ndHole += n2->ndHole;
	}

void Expo::checkSoundnessAndCalcul()
	{
	Node::ndterms++;
	rhoterm = true;
	ndHole = 0;
	n1->checkSoundnessAndCalcul();
	n2->checkSoundnessAndCalcul();
	this->holePosition = n1->holePosition;

	if(n1->ndHole==0)
		{
		this->print(cerr);
		throw SytaxException("Hole not found");
		}
	if(n1->ndHole>1)
		{
		this->print(cerr);
		throw SytaxException("Too much hole");
		}
	if(n1->rhoterm)
	//if(n2->rhoterm)
		{
		this->print(cerr);
		throw SytaxException("Do not tolerate rho-terms in iterated part");
		}

	if(n1->holePosition->empty())
		{
		this->print(cerr);
		throw SytaxException("Do not tolerate single hole in iterated part");
		}

	}

NodeIterator::NodeIterator(Node *n,Node **direct,Mode mode)
	{
	this->mode = mode;
	this->n = n;
	this->direct = direct;
	StackElement e(n,direct);
	stack.push_back(e);
	allowUnderEqual = false;
	}

void NodeIterator::setUnderEqual(bool u)
	{
	allowUnderEqual = u;
	}

const list<unsigned int> &NodeIterator::getPosition()
	{
	return last.position;
	}

Node* NodeIterator::next()
	{
	if(stack.empty())
		return NULL;

	StackElement e;
	if(mode==NodeIterator::BREADTH_FIRST)
		{
		e = stack.front();
		stack.pop_front();
		}
	else
		{
		e = stack.back();
		stack.pop_back();
		}

	last = e;

	if(typeid(*e.n)==typeid(And))
		{
		list<Node*>::iterator it = ((And*)(e.n))->branchs.begin();
		while(it!=((And*)(e.n))->branchs.end())
			{
			stack.push_back(StackElement(*it,&*it));
			it++;
			}
		}
	else if(typeid(*e.n)==typeid(Or))
		{
		list<Node*>::iterator it = ((Or*)(e.n))->branchs.begin();
		while(it!=((Or*)(e.n))->branchs.end())
			{
			stack.push_back(StackElement(*it,&*it));
			it++;
			}
		}
	else if(typeid(*e.n)==typeid(Function))
		{
		vector<Node*>::iterator it = ((Function*)(e.n))->arguments.begin();
		int num = 0;
		while(it!=((Function*)(e.n))->arguments.end())
			{
			stack.push_back(StackElement(*it,&*it,e.position,num));
			num++;
			it++;
			}
		}
	else if(typeid(*e.n)==typeid(Equal))
		{
		if(allowUnderEqual)
			{
			stack.push_back(StackElement(((Equal*)(e.n))->n1,&((Equal*)(e.n))->n1));
			stack.push_back(StackElement(((Equal*)(e.n))->n2,&((Equal*)(e.n))->n2));
			}
		}
	return e.n;
	}

StackElement::StackElement(Node *n,Node **direct)
	{
	this->n = n;
	this->direct = direct;
	}

StackElement::StackElement(Node *n,Node **direct,list<unsigned int> &parentpos,int me)
	{
	this->n = n;
	this->direct = direct;
	position = parentpos;
	position.push_back(me);
	}

StackElement::StackElement()
	{
	}

/**
 * replace le current element by n (you can't use next anymore)
 */
void NodeIterator::replaceBy(Node *n)
	{
	if(last.direct)
		(*last.direct) = n;
	else
		assert(false); // Can't change node
	}

/**
 * come back to the initial possition
 */
void NodeIterator::reset()
	{
	stack.clear();
	StackElement e(n,direct);
	stack.push_back(e);
	}

/**
 * the search for a variable
 */
bool Node::containVar(int idvar)
	{
	return false;
	}

bool Variable::containVar(int idvar)
	{
	return id==idvar;
	}

bool Function::containVar(int idvar)
	{
	vector<Node*>::iterator it = arguments.begin();
	while(it!=arguments.end())
		{
		if((*it)->containVar(idvar))
			return true;
		it++;
		}
	return false;
	}

bool And::containVar(int idvar)
	{
	list<Node*>::iterator it = branchs.begin();
	while(it!=branchs.end())
		{
		if((*it)->containVar(idvar))
			return true;
		it++;
		}
	return false;
	}

bool Or::containVar(int idvar)
	{
	list<Node*>::iterator it = branchs.begin();
	while(it!=branchs.end())
		{
		if((*it)->containVar(idvar))
			return true;
		it++;
		}
	return false;
	}

bool Equal::containVar(int idvar)
	{
	return n1->containVar(idvar) || n2->containVar(idvar);
	}

bool Expo::containVar(int idvar)
	{
	return n1->containVar(idvar) || n2->containVar(idvar);
	}

/**
 * repalce a variable by a generic node
 */
void Node::replace(int idvar,Node *n)
	{
	}

void Function::replace(int idvar,Node *n)
	{
	vector<Node*>::iterator it = arguments.begin();
	while(it!=arguments.end())
		{
		if((typeid(**it)==typeid(Variable))&&(((Variable*)(*it))->id==idvar))
			*it = n->duplicate();
		(*it)->replace(idvar,n);
		it++;
		}
	}

void Or::replace(int idvar,Node *n)
	{
	list<Node*>::iterator it = branchs.begin();
	while(it!=branchs.end())
		{
		if((typeid(**it)==typeid(Variable))&&(((Variable*)(*it))->id==idvar))
			*it = n->duplicate();
		(*it)->replace(idvar,n);
		it++;
		}
	}

void And::replace(int idvar,Node *n)
	{
	list<Node*>::iterator it = branchs.begin();
	while(it!=branchs.end())
		{
		if((typeid(**it)==typeid(Variable))&&(((Variable*)(*it))->id==idvar))
			*it = n->duplicate();
		(*it)->replace(idvar,n);
		it++;
		}
	}

void Equal::replace(int idvar,Node *n)
	{
	if((typeid(*n1)==typeid(Variable))&&(((Variable*)n1)->id==idvar))
		n1 = n->duplicate();
	
	if((typeid(*n2)==typeid(Variable))&&(((Variable*)n2)->id==idvar))
		n2 = n->duplicate();

	n1->replace(idvar,n);
	n2->replace(idvar,n);
	}

void Expo::replace(int idvar,Node *n)
	{
	if((typeid(*n1)==typeid(Variable))&&(((Variable*)n1)->id==idvar))
		n1 = n->duplicate();
	
	if((typeid(*n2)==typeid(Variable))&&(((Variable*)n2)->id==idvar))
		n2 = n->duplicate();

	n1->replace(idvar,n);
	n2->replace(idvar,n);
	}

Node* Hole::duplicate()
	{
	return new Hole();
	}

Node* Clash::duplicate()
	{
	return new Clash();
	}

Node* Trivial::duplicate()
	{
	return new Trivial();
	}

NumericalVar* NumericalVar::duplicate()
	{
	return new NumericalVar(*this);
	}

Condition* CondANplusB::duplicate()
	{
	return new CondANplusB(n1->duplicate(),a,n2->duplicate(),b);
	}

Condition* CondA::duplicate()
	{
	return new CondA(n1->duplicate(),a);
	}

Condition* CondNplusN::duplicate()
	{
	return new CondNplusN(n1->duplicate(),n2->duplicate(),n3->duplicate());
	}

Condition* CondN::duplicate()
	{
	return new CondN(n1->duplicate(),n2->duplicate());
	}

Node* Variable::duplicate()
	{
	return new Variable(*this);
	}

Node* Function::duplicate()
	{
	Function *dme= new Function(id,parser);
	vector<Node*>::iterator it = arguments.begin();
	while(it!=arguments.end())
		{
		dme->addArgument((*it)->duplicate());
		it++;
		}
	return dme;
	}

Node* Or::duplicate()
	{
	Or *dme= new Or();
	list<Node*>::iterator it = branchs.begin();
	while(it!=branchs.end())
		{
		dme->addBranch((*it)->duplicate());
		it++;
		}
	return dme;
	}

Node* And::duplicate()
	{
	And *dme= new And();
	list<Node*>::iterator it = branchs.begin();
	while(it!=branchs.end())
		{
		dme->addBranch((*it)->duplicate());
		it++;
		}
	return dme;
	}

Node* Equal::duplicate()
	{
	return new Equal(n1->duplicate(),n2->duplicate());
	}

Node* Expo::duplicate()
	{
	if(free)
		return new Expo(n1->duplicate(),var->duplicate(),n2->duplicate());
	else
		return new Expo(n1->duplicate(),expNum,n2->duplicate());
	}

/**
* unfold two time the exp term
*/
Node* Expo::buildTwo(Node *sub)
	{
	return buildOne(buildOne(sub));
	}

/**
* unfold one time the exp term
*/
Node* Expo::buildOne(Node *sub)
	{
	if(typeid(*n1)==typeid(Hole))
		return ((sub)?sub:(n2->duplicate()));

	Node *ret = n1->duplicate();

	Node *fonction = ret;
	Node **hole;
	list<unsigned int>::iterator it = holePosition->begin();
	while(it!=holePosition->end())
		{
		assert(typeid(*fonction)==typeid(Function));// This node is not a function
		hole = &((Function*)fonction)->arguments[*it];
		fonction = ((Function*)fonction)->arguments[*it];
		it++;
		}
	assert(typeid(**hole)==typeid(Hole)); //This node is not a hole

	delete fonction;
	*hole = ((sub)?sub:(n2->duplicate()));
	return ret;
	}

/**
* duplicate the tree and replace the node a the position path by sub
*/
Node* Node::buildOneAndReplace(list<unsigned int> &path,Node *sub)
	{
	assert(path.empty());
	return sub;
	}

Node* Function::buildOneAndReplace(list<unsigned int> &path,Node *sub)
	{
	assert(typeid(*this)==typeid(Function));
	if(path.empty())
		return sub;

	unsigned int top = path.back();
	path.pop_back();

	Function *ret = new Function(id,parser);
	vector<Node*>::iterator it = arguments.begin();
	unsigned int num = 0;
	while(it!=arguments.end())
		{
		if(num==top)
			{
			if(path.empty())
				ret->addArgument(sub);
			else
				ret->addArgument(((Function*)(*it))->buildOneAndReplace(path,sub));
			}
		else
			ret->addArgument((*it)->duplicate());
		it++;
		num++;
		}
	return ret;
	}

/**
* search for the node at the position path
*/
Node *Node::find(list<unsigned int> &path)
	{
	assert(path.empty());
	return this;
	}

Node *Function::find(list<unsigned int> &path)
	{
	assert(typeid(*this)==typeid(Function));

	if(path.empty())
		return this;

	unsigned int top = path.back();
	path.pop_back();

	assert(top<arguments.size());
	if(path.empty())
		return arguments[top];
	else
		{
		assert(typeid(*((Function*)arguments[top]))==typeid(Function));
		return ((Function*)arguments[top])->find(path);
		}
	}

/**
* equality of structure
*/
bool Node::isEqual(Node *n)
	{
	return (typeid(*this)==typeid(*n));
	}

bool NumericalVar::isEqual(Node *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	return (this->id==((NumericalVar*)n)->id);
	}

bool Variable::isEqual(Node *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	return (this->id==((Variable*)n)->id);
	}

bool Function::isEqual(Node *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;

	if(this->id!=((Function*)n)->id)
		return false;

	vector<Node*>::iterator it = arguments.begin();
	vector<Node*>::iterator it2 = ((Function*)n)->arguments.begin();
	while(it!=arguments.end())
		{
		if(!(*it)->isEqual(*it2))
			return false;
		it2++;
		it++;
		}

	return true;
	}

bool Or::isEqual(Node *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;

	list<Node*>::iterator it = branchs.begin();
	list<Node*>::iterator it2 = ((Or*)n)->branchs.begin();
	while(it!=branchs.end())
		{
		if(!(*it)->isEqual(*it2))
			return false;
		it2++;
		it++;
		}
	return true;
	}

bool And::isEqual(Node *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	
	list<Node*>::iterator it = branchs.begin();
	list<Node*>::iterator it2 = ((And*)n)->branchs.begin();
	while(it!=branchs.end())
		{
		if(!(*it)->isEqual(*it2))
			return false;
		it2++;
		it++;
		}

	list<Condition*>::iterator it3 = conditions.begin();
	list<Condition*>::iterator it4 = ((And*)n)->conditions.begin();
	while(it3!=conditions.end())
		{
		if(!(*it3)->isEqual(*it4))
			return false;
		it3++;
		it4++;
		}

	return true;
	}

bool Equal::isEqual(Node *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	if(!this->n1->isEqual(((Equal*)n)->n1))
		return false;
	if(!this->n2->isEqual(((Equal*)n)->n2))
		return false;
	return true;
	}

bool Expo::isEqual(Node *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	if(!this->n1->isEqual(((Expo*)n)->n1))
		return false;
	if(!this->n2->isEqual(((Expo*)n)->n2))
		return false;

	if(this->free==((Expo*)n)->free)
	if(free)
		return this->var->id == ((Expo*)n)->var->id;
	else
		return this->expNum == ((Expo*)n)->expNum;

	return false;
	}

bool CondA::isEqual(Condition *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	return false;
	}

bool CondANplusB::isEqual(Condition *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	return false;
	}

bool CondNplusN::isEqual(Condition *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	return false;
	}

bool CondN::isEqual(Condition *n)
	{
	if(typeid(*this)!=typeid(*n))
		return false;
	return false;
	}

Node* Expo::build(int num,Node *sub)
	{
	Node *ret = sub;
	int i;
	for(i=0;i<num;i++)
		ret = buildOne(ret);
	return ret;
	}

