%{

#include "FlexLexer.h"
#include <stdio.h>
#include <string>
#include <vector>
#include "parser.h"
#include "config.h"

int yylex();

extern Node **result;
extern Parser* extParser;
extern void set_extParser(Parser* extP);
extern int yyerror(char *s);
extern void set_result(Node **r);
extern yyFlexLexer* lexer;
extern int numLine;


%}


%defines

%union {
    int integerVal;
    std::string* stringVal;
    Node* node;
    Condition* cond;
    NumericalVar* numvar;
}

%token END_OF_FILE "end of file"
%token EOL "end of line"
%token AND "and"
%token OR "or"
%token EQUAL "equal"
%token EQUAL2 "equal equal"
%token LEFT_PAR "left parenthesis"
%token RIGHT_PAR "right parenthesis"
%token POWER "^ ie power"
%token LEFT_BRA "{ ie left brace"
%token RIGHT_BRA "} ie right braces"
%token HOLE "@ ie hole"
%token COMMA ", ie comma"
%token <stringVal> MAJ "strings beginning by a high case char"
%token <stringVal> MIN "strings beginning by a low case char"
%token <integerVal> INT "integer"
%token PLUS "+"
%token TIMES "*"
%token RUN "run"
%token DOT
%token SET
%token HELP
%token STEP
%token INFO
%token ZERO
%token BEEP

%type <node> T Tand Tor Tf A Input Ligne Tp Tpf Teq
%type <numvar> NumV
%type <cond> Cond
%type <stringVal> FUNC

%start Input

%%

Input:
 		Ligne { if($1) { $$ = new And(); ((And*)$$)->addBranch($1); *result = $$;} else {$$ = NULL; *result = $$; } }
    | Input Ligne { if($1) $$ = $1; else { $$ = new And(); *result = $$;} if($2) ((And*)$$)->addBranch($2); }
    ;

Ligne:
    EOL			 { $$ = NULL; }
  | RUN EOL  { $$ = NULL; return 0;}
  | HELP EOL { Configuration::printConsol(); $$ = NULL; }
  | SET OPTION EOL { $$ = NULL; }
  | T EOL    { $$ = $1;}
  ;
  
OPTION:
		STEP	{ extParser->config->pauseEveryStep = true; }
	|	INFO	{ extParser->config->printRule = true; }
	|	ZERO	{ /* extParser->config->NcanbeZero = true; */ }
	|	BEEP	{ extParser->config->beep = 1; }
	;
	
T:
    Tf
    | Tand AND Tf { $$ = $1; ((And*)$$)->addBranch($3);}
    | Tor OR Tf { $$ = $1; ((Or*)$$)->addBranch($3); }
    /* | Tf EQUAL Tf { $$ = new Equal($1, $3); } */
    ;

Tand:
    Tf { $$ = new And(); ((And*)$$)->addBranch($1); }
    | Cond { $$ = new And(); ((And*)$$)->addCondition($1); }
    | Tand AND Cond { $$ = $1; ((And*)$$)->addCondition($3); }
    | Tand AND Tf { $$ = $1; ((And*)$$)->addBranch($3); }
    ;

Tor:
    Tf { $$ = new Or(); ((Or*)$$)->addBranch($1); }
    | Tor OR Tf { $$ = $1; ((Or*)$$)->addBranch($3);}
    ;

Tf:
    LEFT_PAR T RIGHT_PAR { $$ = $2;}
    | Teq {$$ = $1;}
    ;

Teq:
    Tp EQUAL Tp { $$ = new Equal($1, $3); }

Tp:
    HOLE { $$ = new Hole();}
    | FUNC LEFT_PAR A RIGHT_PAR { $$ = $3; ((Function*)$$)->setName($1->c_str(), extParser);  delete $1;}
    | MIN { $$ = new Function($1->c_str(), extParser ); delete $1;}
    | MAJ { $$ = new Variable($1->c_str(), extParser ); delete $1;}
    | Tp POWER LEFT_BRA NumV RIGHT_BRA DOT Tpf { $$ = new Expo($1,$4,$7); }
    | Tp POWER LEFT_BRA INT RIGHT_BRA DOT Tpf { $$ = new Expo($1,$4,$7); } 
    ;

/* experience pourrie de Gaby pour desambiguifier la grammaire */
Tpf:
    LEFT_PAR Tp RIGHT_PAR { $$ = $2; }
    | FUNC LEFT_PAR A RIGHT_PAR { $$ = $3; ((Function*)$$)->setName($1->c_str(), extParser);  delete $1;}
    | MIN { $$ = new Function($1->c_str(), extParser ); delete $1;}
    | MAJ { $$ = new Variable($1->c_str(), extParser ); delete $1;}
    ;

FUNC:
    MAJ { $$ = $1; }
    | MIN { $$ = $1; }
    ;

A:
    Tp { $$ = new Function(); ((Function*)$$)->addArgument($1);}
    | A COMMA Tp { $$ = $1; ((Function*)$$)->addArgument($3); }
    ;

Cond: 
    NumV EQUAL2 INT TIMES NumV PLUS INT {$$ = new CondANplusB($1, $3, $5, $7); }
    | NumV EQUAL2 INT TIMES NumV { $$ = new CondANplusB($1, $3, $5, 0); }
    | NumV EQUAL2 NumV PLUS NumV { $$ = new CondNplusN($1, $3, $5); }
    | NumV EQUAL2 NumV { $$ = new CondN($1, $3); }
    ;

NumV:
    MAJ { $$ = new NumericalVar($1->c_str(), extParser ,false); delete $1;}
    | MIN { $$ = new NumericalVar($1->c_str(), extParser ,false); delete $1;}
    ;

%%

Parser* extParser;
Node **result;
yyFlexLexer* lexer;
int numLine;

void set_result(Node **r)
	{
	result = r;
	}

void set_extParser(Parser* extP)
	{
	extParser = extP;
	}

int yyerror(char *s)
	{
	printf("\nErreur : %s (line : %i)\n",s,numLine);
	return 1;
	}

int yylex(void) 
	{
  return lexer->yylex(); 
	}
