#include "synnaeve.guillame.bert.h"
#include "unification.h"
#include <assert.h>
#include <iostream>
#include <string>
#include <climits>

namespace Solver
	{

bool Vertex::setBounds(int min,int max)
	{
	bool ch = false;
	if(min>this->min)
		{ this->min = min; ch = true; }
	if(max<this->max)
		{ this->max = max; ch = true; }
	return ch;
	}

bool Vertex::setBoundMin(int min)
	{
	if(min>this->min)
		{ this->min = min; return true; }
	return false;
	}

bool Vertex::setBoundMax(int max)
	{
	if(max<this->max)
		{ this->max = max; return true; }
	return false;
	}

Graph::Graph(Node* node,Unification *unification)
	{
	this->unification = unification;
	solus = new SolverSolution();
	solus->unification = unification;


	// ====== DEBUG =========
	Vertex *v0 = addVertex(0);
	Vertex *v1 = addVertex(1);
	Vertex *v2 = addVertex(2);
	Vertex *v3 = addVertex(3);
	Vertex *v4 = addVertex(4);
	Vertex *v5 = addVertex(5);
	Vertex *v6 = addVertex(-1);
	Vertex *v7 = addVertex(-2);

	addEdge(v0,v1,2,0);

	addEdge(v1,v7,1,0);
	addEdge(v2,v7,1,0);
	addEdge(v3,v7,-1,0);

	addEdge(v0,v6,1,0);
	addEdge(v5,v6,1,0);
	addEdge(v4,v6,-1,0);

	addEdge(v4,v3,1,2);

	success = true;
	solus->ndvar = 0;
	return;

	solus->ndvar = unification->parser->variabletab.size();
	solus->variableSolution = new VariableSolution[solus->ndvar];
	success = explo(node);
	}

Vertex* Graph::addVertex(int id)
	{
	if(id>=0)
		{
		list<Vertex*>::iterator it = vertexs.begin();
		while(it!=vertexs.end())
			{
			if((*it)->variable==id)
				return *it;
			it++;
			}
		}

	Vertex *v = new Vertex();
	v->count = 0;
	v->variable = id;
	v->min = (unification->configuration->NcanbeZero)?0:1;
	v->max = INT_MAX;
	v->father = NULL;
	v->visited = false;
	this->vertexs.push_back(v);
	return v;
	}

Edge* Graph::addEdge(Vertex *v1,Vertex *v2,int a,int b)
	{
	Edge *e = new Edge();
	e->a = a;
	e->b = b;
	e->n1 = v1;
	e->n2 = v2;
	v1->out.push_back(e);
	v2->in.push_back(e);
	this->edges.push_back(e);
	return e;
	}

bool Graph::explo(Node* n)
	{
	bool val = true;
	int first = (unification->configuration->NcanbeZero)?0:1;

	if(typeid(*n)==typeid(Clash))
		val = false;
	else if(typeid(*n)==typeid(Trivial))
		val = true;
	else if(typeid(*n)==typeid(Equal))
		{
		Equal *e = (Equal*)n;
		val = true;
		if(typeid(*e->n1)==typeid(Variable))
			solus->variableSolution[((Variable*)(e->n1))->id].definition = e->n2;
		else
			return false;

		if(!explo(e->n2))
			val = false;
		}
	else if(typeid(*n)==typeid(And))
		{
		And *a = (And*)n;
		val = true;
		list<Node*>::iterator it = a->branchs.begin();
		while(it!=a->branchs.end())
			{
			if(!explo(*it))
				{
				val = false;
				break;
				}
			it++;
			}
		list<Condition*>::iterator it2 = a->conditions.begin();
		while(it2!=a->conditions.end())
			{
			Vertex *v = addVertex((*it2)->n1->id);

			if(typeid(**it2)==typeid(CondANplusB))
				{
				Edge *e = addEdge(addVertex(((CondANplusB*)(*it2))->n2->id),v,((CondANplusB*)(*it2))->a,((CondANplusB*)(*it2))->b);
				if((e->a<0)&&(e->b<=0))
					{
					e->n1->setBoundMax(-(e->b-first)/e->a);
					e->n2->setBoundMax(e->b + e->a*first);
					}
				else
					if((e->a<0)&&(e->b<0))
						if(first==0)
							{
							e->n1->setBounds(0,0);
							e->n2->setBounds(0,0);
							}
						else
							return false;
				}
			else if(typeid(**it2)==typeid(CondNplusN))
				{
				Vertex *onode = addVertex(-1);
				addEdge(v,onode,-1,0);
				addEdge(addVertex(((CondNplusN*)(*it2))->n2->id),onode,1,0);
				addEdge(addVertex(((CondNplusN*)(*it2))->n3->id),onode,1,0);
				}
			else if(typeid(**it2)==typeid(CondN))
				addEdge(addVertex(((CondANplusB*)(*it2))->n2->id),v,1,0);
			else if(typeid(**it2)==typeid(CondA))
				v->setBounds(((CondA*)(*it2))->a,((CondA*)(*it2))->a);
			it2++;
			}
		}
	else if(typeid(*n)==typeid(Function))
		{
		Function *f = (Function*)n;
		vector<Node*>::iterator it = f->arguments.begin();
		val = true;
		while(it!=f->arguments.end())
			{
			if(!explo(*it))
				{
				val = false;
				break;
				}
			it++;
			}
		}
	else if(typeid(*n)==typeid(Or))
		{
		assert(false); // Ceci n'est pas une forme disjonctive (les Ors on dj t traits)
		}
	else if(typeid(*n)==typeid(Expo))
		{
		Expo *e = (Expo*)n;
		val = explo(e->n1);
		if(!explo(e->n2))
			val = false;
		if(e->free)
			addVertex(e->var->id);
			//solution->numericalvariableSolution[e->var->id].seeoutside = true;
		}
	else if(typeid(*n)==typeid(Variable))
		{
		Variable *v = (Variable*)n;
		solus->variableSolution[v->id].seeoutside = true;
		val = true;
		}
	return val;
	}

SolverSolution *Graph::run()
	{
	if(!success)
		{
		solus->success = false;
		return solus;
		}

	cout << "NDNoeuds:" << this->vertexs.size() << endl;
	cout << "NDarcs:" << this->edges.size() << endl;
	cout << endl;

	print();

	list<Vertex*> vertexs;
	list<Edge*> edges;
	while(seekCycle(vertexs,edges))
		{
		cout << "Cycle : " << vertexs.size() << " "<< edges.size() <<endl;
		cout << "   ";
		list<Vertex*>::iterator it;
		it = vertexs.begin();
		while(it!=vertexs.end())
			{
			cout << (*it)->variable;
			it++;
			}
		cout << endl;

		list<Contribution> contribution;
		list<Contribution>::iterator it3;

		list<Edge*>::iterator it2 = edges.begin();
		float a = 1;
		float b = 0;
		Vertex *prec;

		it = vertexs.end();
		it--;
		it--;
		Vertex *cur = *it;

		it = vertexs.begin();
		while(it2!=edges.end())
			{
			prec = cur;
			cur = *it;
			it++;

			if(cur->variable<0)
				{
				list<Edge*>::iterator it4;
				it4 = cur->in.begin();
				while(it4!=cur->in.end())
					{
					if((((*it4)->n1)!=prec)&&(((*it4)->n1)!=(*(it))))
						{
						Contribution tmp;
						tmp.a = (float)(*it4)->a;
						b += (float)(*it4)->b;
						tmp.v = cur;
						contribution.push_back(tmp);
						}
					it4++;
					}

				it4 = cur->out.begin();
				while(it4!=cur->out.end())
					{
					if((((*it4)->n2)!=prec)&&(((*it4)->n2)!=(*(it))))
						{
						Contribution tmp;
						tmp.a = (float)-(*it4)->a;
						b += (float)-(*it4)->b;
						tmp.v = cur;
						contribution.push_back(tmp);
						}
					it4++;
					}
				}

			float way = (cur==(*it2)->n1)?1.f:-1.f;


			if(way==1)
				{
				a *= (*it2)->a;
				b *= (*it2)->a;
				b += (*it2)->b;
				}
			else
				{
				a /= (*it2)->a;
				b /= (*it2)->a;
				b -= (*it2)->b/(*it2)->a;
				}

			it3 = contribution.begin();
			while(it3!=contribution.end())
				{
				if(way==1)
					(*it3).a *= (*it2)->a;
				else
					(*it3).a /= (*it2)->a;
				it3++;
				}

			it2++;
			}
		a--;

		if(contribution.empty())
			{
			if(b!=0)
				{
				solus->success = false;
				return solus;
				}
			if(a!=1)
				{
				solus->success = false;
				return solus;
				}
			else
				{
				if(!unification->configuration->NcanbeZero)
					{
					solus->success = false;
					return solus;
					}
				else
					{
					vertexs.front()->setBounds(0,0);
					//removeEdge(edges.front());
					}
				}
			}
		else
			{
			//Vertex *o = addVertex(-1);
			// TODO
			}

		contribution.clear();
		vertexs.clear();
		edges.clear();
		}

	cout << "Plus de cycle" << endl;
	print();
	
	return solus;
	}

bool SolverSolution::check()
	{
	return success;
	}

void SolverSolution::display(ostream &o)
	{
	unsigned int i;
	for(i=0;i<ndvar;i++)
		{
		o << " " << unification->parser->variabletab[i].name << " = ";
		if(!variableSolution[i].definition)
			o << "*free*";
		else
			variableSolution[i].definition->print(o);
		o << endl;
		}
	}

Graph::~Graph()
	{
	}

bool Graph::seekCycle(list<Vertex*> &ver,list<Edge*> &ed)
	{
	if(vertexs.empty())
		return false;

	resetVisited();

	list<Vertex*> tovisit;
	vertexs.front()->visited = true;
	tovisit.push_back(vertexs.front());
	Vertex *v;
	Vertex *v2;
	Edge *e;

	while(!tovisit.empty())
		{
		v = tovisit.back();
		tovisit.pop_back();

		list<Edge*>::iterator it2;

		it2 = v->in.begin();
		while(it2!=v->in.end())
			{
			if(*it2!=v->father)
				{
				tovisit.push_back((*it2)->n1);
				if((*it2)->n1->visited)
					{
					e = *it2;
					v2 = (*it2)->n1;
					goto buildpath;
					}
				(*it2)->n1->father = *it2;
				(*it2)->n1->visited = true;
				}
			it2++;
			}

		it2 = v->out.begin();
		while(it2!=v->out.end())
			{
			if(*it2!=v->father)
				{
				tovisit.push_back((*it2)->n2);
				if((*it2)->n2->visited)
					{
					e = *it2;
					v2 = (*it2)->n2;
					goto buildpath;
					}
				(*it2)->n2->father = *it2;
				(*it2)->n2->visited = true;
				}
			it2++;
			}
		}
	return false;
buildpath:;

	ed.push_back(e);
	resetVisited();

	Vertex *cur = v2;
	while(cur)
		{
		cur->visited = true;
		if(cur->father)
			cur = (cur==cur->father->n1)?cur->father->n2:cur->father->n1;
		else
			cur = NULL;
		}

	cur = v;
	while(!cur->visited)
		cur = (cur==cur->father->n1)?cur->father->n2:cur->father->n1;

	while(v2!=cur)
		{
		ver.push_front(v2);
		ed.push_front(v2->father);
		v2 = (v2==v2->father->n1)?v2->father->n2:v2->father->n1;
		}

	while(v!=cur)
		{
		ver.push_back(v);
		ed.push_back(v->father);
		v = (v==v->father->n1)?v->father->n2:v->father->n1;
		}

	ver.push_front(cur);
	ver.push_back(cur);
	return true;
	}

void Graph::resetVisited()
	{
	list<Vertex*>::iterator it = vertexs.begin();
	while(it!=vertexs.end())
		(*(it++))->visited = false;
	}

void Graph::print()
	{
	cout << " Vertex:" << endl;
	list<Vertex*>::iterator it = vertexs.begin();
	while(it!=vertexs.end())
		{
		cout << "   " << (*it)->variable << "[" << (*it)->min << "," << (*it)->max << "]" << endl;
		it++;
		}

	cout << " Edges:" << endl;
	list<Edge*>::iterator it2 =  edges.begin();
	while(it2!=edges.end())
		{
		cout << "   " << (*it2)->n1->variable << "\t-[" << (*it2)->a << ";" << (*it2)->b << "]->\t" << (*it2)->n2->variable << endl;
		it2++;
		}
	}

	};

