#include "unification.h"
#include "exception.h"
#include "display.h"
#include <iostream>
#include <typeinfo>
#include <time.h>
#include "synnaeve.guillame.bert.h"
#include "automata.h"
#include <map>
#include <string.h>

#ifdef WIN32
#include <windows.h>
#else

#endif

Unification::Unification(Configuration *configuration,Parser *parser)
	{
	this->configuration = configuration;
	this->parser = parser;
	}

/**
* display current state of the memory
*/
void Unification::debugDisplay(ostream &s)
	{
	if(globconfig->writeMode==Configuration::LATEX)
		s << "$";

	if(typeid(*parser->result)==typeid(And))
		dynamic_cast<And*>(parser->result)->printNoBracket(s);
	else
		parser->result->print(s);

	if(globconfig->writeMode==Configuration::LATEX)
		s << "$\\\\";

	s << endl;

	/*
	// Exemple of iteration
	NodeIterator it(parser->result,&parser->result,NodeIterator::DEPTH_FIRST);
	Node *cur;
	while(cur=it.next())
		{
		s << typeid(*cur).name() << endl;
		}
	*/
	}

/**
* transformation in disjonctive normal form
*/
Node* Unification::SetDisjunctiveSystem(Node *start)
	{
	if(!start)
		start = parser->result;

	Node *ret = start;

	bool action = false;
	bool nothingtodo;
	do
		{
		nothingtodo = true;
		NodeIterator it(ret,&ret,NodeIterator::BREADTH_FIRST);
		Node *cur;

		while((cur=it.next()))
			{
			if(typeid(*cur)==typeid(And))
				{
				list<Node*>::iterator it2 = ((And*)cur)->branchs.begin();
				while(it2!=((And*)cur)->branchs.end())
					{
					if(typeid(*(*it2))==typeid(Or))
						{

						Or *newOr = new Or();
						list<Node*>::iterator it3 = ((Or*)(*it2))->branchs.begin();
						while(it3!=((Or*)(*it2))->branchs.end())
							{
							And *newAnd = new And();
							bool last = ((*it3)==((Or*)(*it2))->branchs.back());

							newAnd->addBranch(*it3);

							//parcour des conditions et ajout
							list<Condition*>::iterator it5 = ((And*)cur)->conditions.begin();
							while(it5!=((And*)cur)->conditions.end())
								{
								newAnd->addCondition((*it5)->duplicate());
								//if(last)
								//	delete (*it5);
								it5++;
								}

							//parcour du And et ajout (sauf it2)
							list<Node*>::iterator it4 = ((And*)cur)->branchs.begin();
							while(it4!=((And*)cur)->branchs.end())
								{
								if(it4!=it2)
									newAnd->addBranch((*it4)->duplicate());
								//if(last)
								//	delete (*it4);
								it4++;
								}

							newOr->addBranch(newAnd);

							if(last)
								break;

							it3++;
							}

						it.replaceBy(newOr);
						action = true;
						nothingtodo = false;
						goto endthisloop;
						}
					it2++;
					}
				}
			}
endthisloop:;
		ret = ApplyR0(ret);
		}
	while(!nothingtodo);
	return ret;
	}

/**
* Application of rules R0 (simplification)
*/
Node* Unification::ApplyR0(Node *start)
	{
	if(!start)
		start = parser->result;

	Node *ret = start;

	bool nothingtodo;
	do
		{
		nothingtodo = true;
		NodeIterator it(ret,&ret,NodeIterator::DEPTH_FIRST);
		it.setUnderEqual(true);
		Node *cur;

		while((cur=it.next()))
			{
			if(typeid(*cur)==typeid(And))
				{
				if((((And*)cur)->branchs.size()==1)&&(((And*)cur)->conditions.empty()))
					{
					it.replaceBy(((And*)cur)->branchs.front());
					delete cur;
					nothingtodo = false;
					goto endthisloop;
					}
				list<Node*>::iterator it2 = ((And*)cur)->branchs.begin();
				while(it2!=((And*)cur)->branchs.end())
					{
					if(typeid(*(*it2))==typeid(And))
						{
						And *subAnd = (And*)(*it2);
						((And*)cur)->branchs.erase(it2);

						list<Node*>::iterator it3 = subAnd->branchs.begin();
						while(it3!=subAnd->branchs.end())
							{
							((And*)cur)->branchs.push_back(*it3);
							it3++;
							}

						list<Condition*>::iterator it4 = subAnd->conditions.begin();
						while(it4!=subAnd->conditions.end())
							{
							((And*)cur)->conditions.push_back(*it4);
							it4++;
							}

						nothingtodo = false;
						goto endthisloop;
						}
					if(typeid(*(*it2))==typeid(Clash))
						{
						it.replaceBy(new Clash());
						nothingtodo = false;
						goto endthisloop;
						}
					if(typeid(*(*it2))==typeid(Trivial))
						{
						((And*)cur)->branchs.erase(it2);
						nothingtodo = false;
						goto endthisloop;
						}
					it2++;
					}
				}

			if(typeid(*cur)==typeid(Expo))
			if(!((Expo*)cur)->free)
				{
				if(((Expo*)cur)->expNum==0)
					{
					// TODO -> delete ((Expo*)cur)->n1
					it.replaceBy(((Expo*)cur)->n2);
					nothingtodo = false;
					goto endthisloop;
					}
				else
					{
					Node *n = ((Expo*)cur)->build(((Expo*)cur)->expNum,NULL);
					// TODO -> delete ((Expo*)cur)
					it.replaceBy(n);
					nothingtodo = false;
					goto endthisloop;
					}
				}

			if(typeid(*cur)==typeid(Or))
				{
				if(((Or*)cur)->branchs.size()==1)
					{
					it.replaceBy(((Or*)cur)->branchs.front());
					delete cur;
					nothingtodo = false;
					goto endthisloop;
					}
				list<Node*>::iterator it2 = ((Or*)cur)->branchs.begin();
				while(it2!=((Or*)cur)->branchs.end())
					{
					if(typeid(*(*it2))==typeid(Or))
						{
						Or *subOr = (Or*)(*it2);
						((Or*)cur)->branchs.erase(it2);

						list<Node*>::iterator it3 = subOr->branchs.begin();
						while(it3!=subOr->branchs.end())
							{
							((Or*)cur)->branchs.push_back(*it3);
							it3++;
							}

						nothingtodo = false;
						goto endthisloop;
						}
					if(typeid(*(*it2))==typeid(Trivial))
						{
						it.replaceBy(new Clash());
						nothingtodo = false;
						goto endthisloop;
						}
					if(typeid(*(*it2))==typeid(Clash))
						{
						((And*)cur)->branchs.erase(it2);
						nothingtodo = false;
						goto endthisloop;
						}
					it2++;
					}
				}
			}
endthisloop:;
		}
	while(!nothingtodo);
	return ret;
	}

/**
* Application of rules R1 (basic unification rules)
*/
bool Unification::tryApplyR1()
	{
	NodeIterator it(parser->result,&parser->result,NodeIterator::DEPTH_FIRST);
	Node *cur;

	// x=x (TODO : pour s=s) -> <triv>
	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			Equal *e = (Equal*)cur;

			/*if((typeid(*e->n1)==typeid(Variable))&&
				(typeid(*e->n2)==typeid(Variable))&&
				(((Variable*)(e->n1))->id == ((Variable*)(e->n2))->id))
				*/
			if(e->n1->isEqual(e->n2))
					{
					if(configuration->printRule)
						{
						Display::texHspace(*mycout);
						(*mycout) << "R1 : x = x " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow \\top":"-> T") << endl;
						/*
						(*mycout) << "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n1->print((*mycout));
						Display::texEndl(*mycout);
						(*mycout) << endl;
						*/
						}
					it.replaceBy(new Trivial());
					return true;
					}
			}

	// s = x  et s non var -> x = s
	it.reset();
	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			Equal *e = (Equal*)cur;
			if((typeid(*e->n1)!=typeid(Variable))&&
				(typeid(*e->n2)==typeid(Variable)))
					{
					if(configuration->printRule)
						{
						Display::texHspace(*mycout);
						(*mycout) << "R1 : s = x " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow":"->") << " x = s" << endl;
						/*
						(*mycout) << "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n1->print((*mycout));
						(*mycout) << endl << "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n2->print((*mycout));
						(*mycout) << endl;
						Display::texEndl(*mycout);*/
						}
					Node *swap = e->n1;
					e->n1 = e->n2;
					e->n2 = swap;
					return true;
					}
			}

	// f(x)=f(y) -> x=y
	it.reset();
	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			Equal *e = (Equal*)cur;
			if((typeid(*e->n1)==typeid(Function))&&
				(typeid(*e->n2)==typeid(Function))&&
				(((Function*)(e->n1))->id == ((Function*)(e->n2))->id))
					{

					if(configuration->printRule)
						{
						Display::texHspace(*mycout);
						(*mycout) << "R1 : f(x)=f(y) " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow":"->") << " x=y" << endl;
						/*
						(*mycout) << "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n1->print((*mycout));
						(*mycout) << endl << "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n2->print((*mycout));
						(*mycout) << endl;
						Display::texEndl(*mycout);*/
						}
					And *add = new And();

					vector<Node*>::iterator it1 = ((Function*)(e->n1))->arguments.begin();
					vector<Node*>::iterator it2 = ((Function*)(e->n2))->arguments.begin();
					while(it1!=((Function*)(e->n1))->arguments.end())
						{
						add->addBranch(new Equal(*it1,*it2));
						it1++;
						it2++;
						}

					it.replaceBy(add);
					return true;
					}
			}


	// f(x)=g(y) -> <clash>
	it.reset();
	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			Equal *e = (Equal*)cur;
			if((typeid(*e->n1)==typeid(Function))&&
				(typeid(*e->n2)==typeid(Function))&&
				(((Function*)(e->n1))->id != ((Function*)(e->n2))->id))
					{
					if(configuration->printRule)
						{
						Display::texHspace(*mycout);
						(*mycout) << "R1 : f(x)=g(y) " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow \\bot":"-><clash>")  << endl;
						/*
						(*mycout) << "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n1->print((*mycout));
						(*mycout) << endl<< "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n2->print((*mycout));
						Display::texEndl(*mycout);
						(*mycout) << endl;*/
						}
					it.replaceBy(new Clash());
					return true;
					}
			}

	// x=s^P et x !E s et x E P et (si s est une var => s E P) -> x=s^P{x->s}
	it.reset();
	while((cur=it.next()))
		if(typeid(*cur)==typeid(And))
			{
			And *andd = (And*)cur;

			list<Node*>::iterator it1 = andd->branchs.begin();
			while(it1!=andd->branchs.end())
				{
				list<Node*>::iterator it2 = andd->branchs.begin();
				while(it2!=andd->branchs.end())
					{
					if((*it1)!=(*it2))
						{
						if((typeid(**it1)==typeid(Equal))&&
							(typeid(*((Equal*)(*it1))->n1)==typeid(Variable)))
							{
							Variable *x = (Variable*)((Equal*)(*it1))->n1;
							Node *s = ((Equal*)(*it1))->n2;
							Node *p = *it2;
							if((!s->containVar(x->id))&&
								(p->containVar(x->id))&&
								((typeid(*s)!=typeid(Variable))||(p->containVar(((Variable*)(s))->id))))
								{
								if(configuration->printRule)
									{
									Display::texHspace(*mycout);
									(*mycout) << "R1 : x=s^P " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow":"->") << " x=s^P{x->s}" << endl;
									Display::texEndl(*mycout);
									}
								p->replace(x->id,s);
								return true;
								}
							}
						}
					it2++;
					}
				it1++;
				}
			}

	// x=s et x E s -> <clash>
	it.reset();
	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			Equal *e = (Equal*)cur;
			if((typeid(*e->n1)==typeid(Variable))&&
				(e->n2->containVar(((Variable*)(e->n1))->id)))
					{
					if(configuration->printRule)
						{
						Display::texHspace(*mycout);
						(*mycout) << "R1 : x=s et x " << ((configuration->writeMode==Configuration::LATEX)?"\\in s \\rightarrow \\bot":"E s -> <clash>") << endl;
						/*
						(*mycout) << "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n1->print((*mycout));
						(*mycout) << endl<< "\t";
						Display::texEndl(*mycout);
						Display::texHspace(*mycout);
						e->n2->print((*mycout));
						(*mycout) << endl;
						Display::texEndl(*mycout);*/
						}
					it.replaceBy(new Clash());
					return true;
					}
			}
	return false;
	}

/**
* Application of rules R2 (Unfold 1)
*/
bool Unification::tryApplyR2()
	{
	NodeIterator it(parser->result,&parser->result,NodeIterator::DEPTH_FIRST);
	Node *cur;
	// s = t[@]^N_p.u -> ...
	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			Equal *e = (Equal*)cur;
			Node *s;
			Node *other;
			int way = 0;
			if((typeid(*(other=e->n2))==typeid(Expo))&&(typeid(*e->n1)!=typeid(Variable)))
				s=e->n1;
			else if((typeid(*(other=e->n1))==typeid(Expo))&&(typeid(*e->n2)!=typeid(Variable)))
				{
				s=e->n2;
				way = 1;
				}
			else continue;
rerun:;
			Node *curSearchExpo = s;
			list<unsigned int>::iterator it2 = other->holePosition->begin();
			bool fail = false;

			if(typeid(*s)==typeid(Expo))
				fail = true;
			else
				while(it2!=other->holePosition->end())
					{
					if(typeid(*curSearchExpo)!=typeid(Function))
						break;
					if(((Function*)curSearchExpo)->arguments.size()<=(*it2))
						break;
					curSearchExpo = ((Function*)curSearchExpo)->arguments[*it2];
					if(typeid(*curSearchExpo)==typeid(Expo))
						{
						fail = true;
						break;
						}
					it2++;
					}

			if(fail)
				{
				if(way==0)
				if((typeid(*(other=e->n1))==typeid(Expo))&&(typeid(*e->n2)!=typeid(Variable)))
						{
						s=e->n2;
						way = 1;
						goto rerun;
						}
				continue;
				}

			if(configuration->printRule)
				{
				Display::texHspace(*mycout);
				(*mycout) << "R2 (Unfold 1) : s = t[@]^N_p.u " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow":"->") << " ..." << endl;
				/*
				(*mycout) << "\t";
				Display::texEndl(*mycout);
				Display::texHspace(*mycout);
				cur->print((*mycout));
				Display::texEndl(*mycout);
				(*mycout) << endl;
				*/
				}

			And *add2 = NULL;
			And *add3 = NULL;
			int num = (configuration->NcanbeZero)?0:1;
			Expo *expoterm = (Expo*)other;

			if((expoterm->free)||(expoterm->expNum==num))
				{
				add2 = new And();
				if(expoterm->free)
					add2->addCondition(new CondA(expoterm->var->duplicate(),num));

				if(configuration->NcanbeZero)
					add2->addBranch(new Equal(s->duplicate(),expoterm->n1->duplicate()));
				else
					add2->addBranch(new Equal(s->duplicate(),expoterm->buildOne()));
				}

			if((expoterm->free)||(expoterm->expNum>1))
				{
				add3 = new And();
				Expo *newExpo = (Expo *)expoterm->duplicate();

				if(newExpo->free)
					{
					NumericalVar *var = new NumericalVar(parser->genNumericalValName().c_str(),parser);
					add3->addCondition(new CondANplusB(expoterm->var->duplicate(),1,var,1));
					delete newExpo->var;
					newExpo->var = var->duplicate();
					}
				else
					newExpo->expNum--;

				add3->addBranch(new Equal(s->duplicate(),expoterm->buildOne(newExpo)));
				}


			if((add2)&&(!add3))
				it.replaceBy(add2);
			else if((add3)&&(!add2))
				it.replaceBy(add3);
			else
				{
				Or *add = new Or();
				add->addBranch(add2);
				add->addBranch(add3);
				it.replaceBy(add);
				}
			return true;
			}

	return false;
	}

int Unification::gcd(unsigned int a, unsigned int b)
	{
	return ( b != 0 ? gcd(b, a % b) : a );
	}

/**
* is l1 a prefix of l2 ?
*/
bool Unification::prefix(const list<unsigned int> &l1,const list<unsigned int> &l2)
	{
	if(l1.size()>l2.size())
		return false;

	list<unsigned int>::const_iterator itp = l1.begin();
	list<unsigned int>::const_iterator itq2 = l2.begin();

	while(itp!=l1.end())
		{
		if((*itp)!=(*itq2))
			return false;
		itq2++;
		itp++;
		}
	return true;
	}

/**
* merging of l1 and l2
*/
list<unsigned int> Unification::append(const list<unsigned int> &l1,const list<unsigned int> &l2)
	{
	list<unsigned int> ret = l1;
	list<unsigned int>::const_iterator it = l2.begin();
	while(it!=l2.end())
		{
		ret.push_back(*it);
		it++;
		}
	return ret;
	}

/**
* Application of rules R3 (Unfold 2)
*/
bool Unification::tryApplyR3()
	{
	NodeIterator it(parser->result,&parser->result,NodeIterator::DEPTH_FIRST);
	Node *cur;
	// s = t[@]^N_p.u -> ...
	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			int way = 0;
			while(way<2)
				{
				way++;

				Equal *e = (Equal*)cur;
				Node *u;
				Node *v;

				if((typeid(*(v=e->n2))==typeid(Expo))&&((u=e->n1)->rhoterm));
				else if((typeid(*(v=e->n1))==typeid(Expo))&&((u=e->n2)->rhoterm));
				else continue;

				NodeIterator it2(u,NULL,NodeIterator::DEPTH_FIRST);
				Node *cur2;
				while((cur2=it2.next()))
				if(typeid(*cur2)==typeid(Expo))
					{
					const list<unsigned int> p = it2.getPosition();

					assert(((Expo*)cur2)->holePosition);
					const list<unsigned int> &q1 = *((Expo*)cur2)->holePosition;
					const list<unsigned int> &q2 = *v->holePosition;

					if((q1.size()!=q2.size())&&(prefix(p,q2)))
						{

						int i;
						int d = gcd((unsigned int)q1.size(),(unsigned int)q2.size());
						int alpha1 = (unsigned int)q1.size() / d;
						int alpha2 = (unsigned int)q2.size() / d;
						//int m = alpha1 * alpha2 * d;

						if(configuration->printRule)
							{
							Display::texHspace(*mycout);
							(*mycout) << "R3 (Unfold 2) : s[t1[@]^N1_q1.u1]_p = t2[@]^N2_q2.u2 " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow":"->") << " ..." << endl;
							/*
							(*mycout) << "\t";
							Display::texEndl(*mycout);
							Display::texHspace(*mycout);
							cur->print((*mycout));
							Display::texEndl(*mycout);
							(*mycout) << endl;
							*/
							// Debug
							/*
							(*mycout) << "p:" << p << endl;
							(*mycout) << "q1:" << q1 << endl;
							(*mycout) << "q2:" << q2 << endl;
							(*mycout) << "d:" << d << endl;
							(*mycout) << endl;
							*/
							}

						Or *addOr = new Or();

						int first = (this->configuration->NcanbeZero)?0:1;

						int r1,r2;
						for(r1=first;r1<alpha2;r1++)
							{
							if(((Expo*)cur2)->free)
								{
								And *addAnd = new And();
								addAnd->addCondition(new CondA(((Expo*)cur2)->var->duplicate(),r1));
								Node *newTerm = u->duplicate();

								Node *curModificationExpNum = newTerm;
								list<unsigned int>::const_iterator it3 = p.begin();
								while(it3!=p.end())
									{
									assert(typeid(*curModificationExpNum)==typeid(Function));
									curModificationExpNum = ((Function*)curModificationExpNum)->arguments[*it3];
									it3++;
									}

								assert(typeid(*curModificationExpNum)==typeid(Expo));

								if(((Expo*)curModificationExpNum)->free)
									{
									delete ((Expo*)curModificationExpNum)->var;
									((Expo*)curModificationExpNum)->free = false;
									}
								((Expo*)curModificationExpNum)->expNum = r1;

								addAnd->addBranch(new Equal(newTerm,v->duplicate()));
								addOr->addBranch(addAnd);
								}
							else if(((Expo*)cur2)->expNum==r1)
								{
								And *addAnd = new And();
								addAnd->addBranch(new Equal(u->duplicate(),v->duplicate()));
								addOr->addBranch(addAnd);
								}
							}

						for(r2=first;r2<alpha1;r2++)
							{
							if(((Expo*)v)->free)
								{
								And *addAnd = new And();
								addAnd->addCondition(new CondA(((Expo*)v)->var->duplicate(),r2));

								Node *newTerm = v->duplicate();
								if(((Expo*)newTerm)->free)
									{
									delete ((Expo*)newTerm)->var;
									((Expo*)newTerm)->free = false;
									}
								((Expo*)newTerm)->expNum = r2;

								addAnd->addBranch(new Equal(u->duplicate(),newTerm));
								addOr->addBranch(addAnd);
								}
							else if(((Expo*)v)->expNum==r2)
								{
								And *addAnd = new And();
								addAnd->addBranch(new Equal(u->duplicate(),v->duplicate()));
								addOr->addBranch(addAnd);
								}
							}

						for(r2=0;r2<alpha1;r2++)
						for(r1=0;r1<alpha2;r1++)
							{

							And *addAnd = new And();
							bool f1 = ((Expo*)cur2)->free;
							bool f2 = ((Expo*)v)->free;

							NumericalVar *m1;
							NumericalVar *m2;

							int m1p;
							int m2p;

							if(f1)
								{
								m1 = new NumericalVar(parser->genNumericalValName().c_str(),parser);
								addAnd->addCondition(new CondANplusB(((Expo*)cur2)->var->duplicate(),alpha2,m1,r1));
								}
							else
								{
								m1p = (((Expo*)cur2)->expNum - r1);
								if((m1p%alpha2) != 0)
									{
									delete addAnd;
									continue;
									}
								m1p /= alpha2;
								if(m1p<first)
									{
									delete addAnd;
									continue;
									}
								}

							if(f2)
								{
								m2 = new NumericalVar(parser->genNumericalValName().c_str(),parser);
								addAnd->addCondition(new CondANplusB(((Expo*)v)->var->duplicate(),alpha1,m2,r2));
								}
							else
								{
								m2p = (((Expo*)v)->expNum - r2) / alpha1;
								if((m2p%alpha1) != 0)
									{
									if(f1)
										delete m1;
									delete addAnd;
									continue;
									}
								m2p /= alpha1;
								if(m2p<first)
									{
									if(f1)
										delete m1;
									delete addAnd;
									continue;
									}
								}

							Node *t1 = ((Expo*)cur2)->n2->duplicate();
							for(i=0;i<r1;i++)
								t1 = ((Expo*)cur2)->buildOne(t1);

							Node *t2 = ((Expo*)v)->n2->duplicate();
							for(i=0;i<r2;i++)
								t2 = ((Expo*)v)->buildOne(t2);

							Node *h1 = ((Expo*)cur2)->n1->duplicate();
							for(i=0;i<alpha2-1;i++)
								h1 = ((Expo*)cur2)->buildOne(h1);

							Node *h2 = ((Expo*)v)->n1->duplicate();
							for(i=0;i<alpha1-1;i++)
								h2 = ((Expo*)v)->buildOne(h2);

							list<unsigned int> tmp = p;

							Node *k;
							if(f1)
								k = ((Function*)u)->buildOneAndReplace(tmp, new Expo(h1,m1,t1));
							else
								k = ((Function*)u)->buildOneAndReplace(tmp, new Expo(h1,m1p,t1));

							if(f2)
								addAnd->addBranch(new Equal(k,new Expo(h2,m2,t2)));
							else
								addAnd->addBranch(new Equal(k,new Expo(h2,m2p,t2)));

							addOr->addBranch(addAnd);
							}
						it.replaceBy(addOr);
						return true;
						}
					}
				}
			}
	return false;
	}

/**
* Application of rules R4 (Unfold 3)
*/
bool Unification::tryApplyR4()
	{
	NodeIterator it(parser->result,&parser->result,NodeIterator::DEPTH_FIRST);
	Node *cur;

	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			int way = 0;
			while(way<2)
				{
				way++;

				Equal *e = (Equal*)cur;
				Node *u;
				Node *v;

				if((typeid(*(v=e->n2))==typeid(Expo))&&((u=e->n1)->rhoterm));
				else if((typeid(*(v=e->n1))==typeid(Expo))&&((u=e->n2)->rhoterm));
				else continue;

				NodeIterator it2(u,NULL,NodeIterator::DEPTH_FIRST);
				Node *cur2;
				while((cur2=it2.next()))
				if(typeid(*cur2)==typeid(Expo))
					{
					const list<unsigned int> p = it2.getPosition();
					const list<unsigned int> &q1 = *((Expo*)cur2)->holePosition;
					const list<unsigned int> &q2 = *v->holePosition;

					if((prefix(p,q2))&&((!prefix(q2,append(p,q1)))||(!prefix(append(p,q1),append(q2,q2)))))
						{

						if(configuration->printRule)
							{
							Display::texHspace(*mycout);
							(*mycout) << "R4 (Unfold 3) : s[t1[@]^N1_q1.u1]_p = t2[@]^N2_q2.u2 " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow":"->") << " ..." << endl;
							/*
							(*mycout) << "\t";
							Display::texEndl(*mycout);
							Display::texHspace(*mycout);
							cur->print((*mycout));
							Display::texEndl(*mycout);
							(*mycout) << endl;
							*/

							// Debug
							/*
							(*mycout) << "p:" << p << endl;
							(*mycout) << "q1:" << q1 << endl;
							(*mycout) << "q2:" << q2 << endl;
							(*mycout) << endl;
							*/
							}

						int first = (configuration->NcanbeZero)?0:1;

						assert(!configuration->NcanbeZero);// pas encore fait

						Or *addOr = new Or();

						if(((Expo*)v)->free)
							{
							And *addAnd = new And();
							addAnd->addCondition(new CondA(((Expo*)v)->var->duplicate(),1));

							addAnd->addBranch(new Equal(((Expo*)v)->buildOne(),((Expo*)u)->duplicate()));
							addOr->addBranch(addAnd);
							}
						else
							{
							if(((Expo*)v)->expNum==1)
								{
								And *addAnd = new And();
								addAnd->addBranch(new Equal(((Expo*)v)->buildOne(),((Expo*)u)->duplicate()));
								addOr->addBranch(addAnd);
								}
							}

						if(((Expo*)cur2)->free)
							{
							And *addAnd = new And();
							addAnd->addCondition(new CondA(((Expo*)cur2)->var->duplicate(),1));

							assert(typeid(*u)==typeid(Function));

							list<unsigned int> tmp = p;
							addAnd->addBranch(new Equal(((Function*)u)->buildOneAndReplace(tmp,((Expo*)cur2)->buildOne()),((Expo*)v)->duplicate()));
							addOr->addBranch(addAnd);
							}
						else
							{
							if(((Expo*)cur2)->expNum==1)
								{
								And *addAnd = new And();
								list<unsigned int> tmp = p;
								addAnd->addBranch(new Equal(((Function*)u)->buildOneAndReplace(tmp,((Expo*)cur2)->buildOne()),((Expo*)v)->duplicate()));
								addOr->addBranch(addAnd);
								}
							}

						if(((Expo*)v)->free)
							{
							And *addAnd = new And();
							addAnd->addCondition(new CondA(((Expo*)v)->var->duplicate(),2));

							addAnd->addBranch(new Equal(((Expo*)v)->buildTwo(),((Expo*)u)->duplicate()));
							addOr->addBranch(addAnd);
							}
						else
							{
							if(((Expo*)v)->expNum==2)
								{
								And *addAnd = new And();
								addAnd->addBranch(new Equal(((Expo*)v)->buildOne(),((Expo*)u)->duplicate()));
								addOr->addBranch(addAnd);
								}
							}

						bool f1 = ((Expo*)cur2)->free;
						bool f2 = ((Expo*)v)->free;

						And *addAnd = new And();

						NumericalVar *m1 = NULL;
						int m1i;
						if(f1)
							{
							m1 = new NumericalVar(parser->genNumericalValName().c_str(),parser);
							addAnd->addCondition(new CondANplusB(((Expo*)cur2)->var->duplicate(),1,m1,1));
							}
						else
							{
							m1i = ((Expo*)cur2)->expNum-1;
							if(m1i<first)
								{
								delete addAnd;
								continue;
								}
							}

						NumericalVar *m2;
						int m2i;
						if(f2)
							{
							m2 = new NumericalVar(parser->genNumericalValName().c_str(),parser);
							addAnd->addCondition(new CondANplusB(((Expo*)v)->var->duplicate(),1,m2,2));
							}
						else
							{
							m2i = ((Expo*)v)->expNum-2;
							if(m2i<first)
								{
								if(f1)
									{
									delete addAnd->conditions.front();
									assert(m1);
									delete m1;
									}
								delete addAnd;
								continue;
								}
							}

						Expo *h1 = (Expo*)v->duplicate();
						if(f2)
							{
							delete h1->var;
							h1->var = m2;
							}
						else
							h1->expNum = m2i;

						Expo *h2 = (Expo*)cur2->duplicate();
						if(f1)
							{
							delete h2->var;
							h2->var = m1;
							}
						else
							h2->expNum = m1i;

						list<unsigned int> tmp = p;
						addAnd->addBranch(new Equal(((Function*)u)->buildOneAndReplace(tmp,((Expo*)cur2)->buildOne(h2)),((Expo*)v)->buildTwo(h1)));
						addOr->addBranch(addAnd);

						if(addOr->branchs.empty())
							{
							delete addOr;
							continue;
							}

						it.replaceBy(addOr);
						return true;
						}
					}
				}
			}
	return false;
	}

/**
* Application of rules R5 (Decompose 2)
*/
bool Unification::tryApplyR5()
	{
	NodeIterator it(parser->result,&parser->result,NodeIterator::DEPTH_FIRST);
	Node *cur;

	while((cur=it.next()))
		if(typeid(*cur)==typeid(Equal))
			{
			Equal *e = (Equal*)cur;
			Node *u;
			Node *v;

			int way = 0;
			if((typeid(*(v=e->n2))==typeid(Expo))&&((u=e->n1)->rhoterm))
				;
			else if((typeid(*(v=e->n1))==typeid(Expo))&&((u=e->n2)->rhoterm))
				way = 1;
			else continue;
rerun:;

			NodeIterator it2(u,NULL,NodeIterator::DEPTH_FIRST);
			Node *cur2;
			while((cur2=it2.next()))
			if(typeid(*cur2)==typeid(Expo))
				{
				const list<unsigned int> p = it2.getPosition();
				const list<unsigned int> &q1 = *((Expo*)cur2)->holePosition;
				//const list<unsigned int> &q2 = *v->holePosition;

				if(configuration->printRule)
					{
					Display::texHspace(*mycout);
					(*mycout) << "R5 (Decompose 2) : s[v1[w1[@]_p]^N1_q.u1]_p = (v2[w2[@]_q]_p)^N2.u2 " << ((configuration->writeMode==Configuration::LATEX)?"\\rightarrow":"->") << " ..." << endl;
					/*(*mycout) << "\t";
					Display::texEndl(*mycout);
					Display::texHspace(*mycout);
					cur->print((*mycout));
					Display::texEndl(*mycout);
					(*mycout) << endl;
					*/
					// Debug
					/*
					(*mycout) << "p:" << p << endl;
					(*mycout) << "q1:" << q1 << endl;
					(*mycout) << "q2:" << q2 << endl;
					(*mycout) << endl;
					*/
					}

				list<unsigned int> qprim;
				list<unsigned int>::const_iterator it3 = q1.begin();
				int num = q1.size() - p.size();
				for(;num>0;num--)
					{
					qprim.push_back(*it3);
					it3++;
					}

				And *addAnd = new And();

				Function *addA = new Function(parser->genContstantValName().c_str(),parser);

				list<unsigned int>tmp1;
				list<unsigned int>tmp2;
				list<unsigned int>tmp3;

				Node *w1 = (((Expo*)cur2)->n1)->find(tmp1=qprim);
				Node *w2 = (((Expo*)v)->n1)->find(tmp1=p);

				addAnd->addBranch(new Equal(
					u->buildOneAndReplace(tmp1=p,addA->duplicate()),
					w1->buildOneAndReplace(tmp3=p,addA->duplicate())
					));

				addAnd->addBranch(new Equal(
					u->buildOneAndReplace(tmp1=p,addA->duplicate()),
					((Expo*)v)->n1->buildOneAndReplace(tmp2=p,addA->duplicate())
					));

				addAnd->addBranch(new Equal(
					((Expo*)cur2)->n1->buildOneAndReplace(tmp1=qprim,addA->duplicate()),
					w2->buildOneAndReplace(tmp2=qprim,addA->duplicate())
					));

				Or *addOr = new Or();

				bool buildit = true;

				And *addAnd2 = new And();
				if((!((Expo*)cur2)->free)&&(!((Expo*)v)->free))
					{
					if(((Expo*)cur2)->expNum==((Expo*)v)->expNum)
						;
					else
						buildit= false;
					}
				else
					{
					if((((Expo*)cur2)->free)&&(((Expo*)v)->free))
						addAnd2->addCondition(new CondN(((Expo*)cur2)->var->duplicate(),((Expo*)v)->var->duplicate()));
					else if(!((Expo*)cur2)->free)
						addAnd2->addCondition(new CondA(((Expo*)v)->var->duplicate(),((Expo*)cur2)->expNum));
					else
						addAnd2->addCondition(new CondA(((Expo*)cur2)->var->duplicate(),((Expo*)v)->expNum));
					}

				if(buildit)
					{
					addAnd2->addBranch(new Equal(u->buildOneAndReplace(tmp1=p,((Expo*)cur2)->n2->duplicate()),((Expo*)v)->n2->duplicate()));
					addOr->addBranch(addAnd2);
					}
				else
					delete addAnd2;

				int first = (this->configuration->NcanbeZero)?0:1;

				addAnd2 = new And();
				int m1i;
				NumericalVar *m1 = NULL;
				if((!((Expo*)cur2)->free)&&(!((Expo*)v)->free))
					{
					m1i = ((Expo*)cur2)->expNum - ((Expo*)v)->expNum;
					if(m1i>=first)
						;
					else
						buildit= false;
					}
				else
					{
					m1 = new NumericalVar(parser->genNumericalValName().c_str(),parser);

					if((((Expo*)cur2)->free)&&(((Expo*)v)->free))
						addAnd2->addCondition(new CondNplusN(((Expo*)cur2)->var->duplicate(),((Expo*)v)->var->duplicate(),m1->duplicate()));
					else if(!((Expo*)cur2)->free)
						addAnd2->addCondition(new CondANplusB(((Expo*)v)->var->duplicate(),-1,m1->duplicate(),((Expo*)cur2)->expNum));
					else
						addAnd2->addCondition(new CondANplusB(((Expo*)cur2)->var->duplicate(),1,m1->duplicate(),((Expo*)v)->expNum));
					}

				if(buildit)
					{
					Node *h1 = u->duplicate();

					Expo *coup = (Expo*)h1->find(tmp1=p);
					assert(typeid(*coup)==typeid(Expo));

					coup->free = (m1!=NULL);
					if(m1)
						coup->var = m1;
					else
						coup->expNum = m1i;

					addAnd2->addBranch(new Equal(h1,((Expo*)v)->n2));
					addOr->addBranch(addAnd2);
					}
				else
					delete addAnd2;

				addAnd2 = new And();
				m1 = NULL;
				if((!((Expo*)cur2)->free)&&(!((Expo*)v)->free))
					{
					m1i = ((Expo*)v)->expNum - ((Expo*)cur2)->expNum;
					if(m1i>=first)
						;
					else
						buildit= false;
					}
				else
					{
					m1 = new NumericalVar(parser->genNumericalValName().c_str(),parser);

					if((((Expo*)v)->free)&&(((Expo*)cur2)->free))
						addAnd2->addCondition(new CondNplusN(((Expo*)v)->var->duplicate(),((Expo*)cur2)->var->duplicate(),m1->duplicate()));
					else if(!((Expo*)cur2)->free)
						addAnd2->addCondition(new CondANplusB(((Expo*)cur2)->var->duplicate(),-1,m1->duplicate(),((Expo*)v)->expNum));
					else
						addAnd2->addCondition(new CondANplusB(((Expo*)v)->var->duplicate(),1,m1->duplicate(),((Expo*)cur2)->expNum));
					}

				if(buildit)
					{
					Expo *h1 = (Expo*)v->duplicate();

					h1->free = (m1!=NULL);
					if(m1)
						h1->var = m1;
					else
						h1->expNum = m1i;

					addAnd2->addBranch(new Equal(w1->buildOneAndReplace(tmp1=p,((Expo*)cur2)->n2->duplicate()),h1));
					addOr->addBranch(addAnd2);
					}
				else
					delete addAnd2;

				if(addOr->branchs.empty())
					delete addOr;
				else
					addAnd->addBranch(addOr);

				//it.replaceBy(SetDisjunctiveSystem((Node*)addAnd));
				it.replaceBy((Node*)addAnd);
				return true;
				}

			if(way==0)
			if((typeid(*(v=e->n1))==typeid(Expo))&&((u=e->n2)->rhoterm))
					{
					way = 1;
					goto rerun;
					}
			}

	return false;
	}

/**
* Explore and check definition (is the current system well define variables)
*/
bool Unification::exploDefinition(Node *n,SimpleSolution *solution)
	{
	bool val = true;

	if(typeid(*n)==typeid(Clash))
		val = false;
	else if(typeid(*n)==typeid(Trivial))
		val = true;
	else if(typeid(*n)==typeid(Equal))
		{
		Equal *e = (Equal*)n;
		val = true;
		if(typeid(*e->n1)==typeid(Variable))
			solution->variableSolution[((Variable*)(e->n1))->id].definition = e->n2;
		else
			return false;
		if(!exploDefinition(e->n2,solution))
			val = false;
		}
	else if(typeid(*n)==typeid(And))
		{
		And *a = (And*)n;
		val = true;
		list<Node*>::iterator it = a->branchs.begin();
		while(it!=a->branchs.end())
			{
			if(!exploDefinition(*it,solution))
				{
				val = false;
				break;
				}
			it++;
			}
		list<Condition*>::iterator it2 = a->conditions.begin();
		while(it2!=a->conditions.end())
			{

			/*
			// multi definition
			if(solution->numericalvariableSolution[(*it2)->n1->id].definitionc)
				{
				val = false;
				break;
				}
			else
			*/

			solution->numericalvariableSolution[(*it2)->n1->id].definitionc.push_back(*it2);

			if(typeid(**it2)==typeid(CondANplusB))
				solution->numericalvariableSolution[((CondANplusB*)(*it2))->n2->id].seeoutside = true;
			else if(typeid(**it2)==typeid(CondNplusN))
				{
				solution->numericalvariableSolution[((CondNplusN*)(*it2))->n2->id].seeoutside = true;
				solution->numericalvariableSolution[((CondNplusN*)(*it2))->n3->id].seeoutside = true;
				}
			else if(typeid(**it2)==typeid(CondN))
				solution->numericalvariableSolution[((CondN*)(*it2))->n2->id].seeoutside = true;

			it2++;
			}
		}
	else if(typeid(*n)==typeid(Function))
		{
		Function *f = (Function*)n;
		vector<Node*>::iterator it = f->arguments.begin();
		val = true;
		while(it!=f->arguments.end())
			{
			if(!exploDefinition(*it,solution))
				{
				val = false;
				break;
				}
			it++;
			}
		}
	else if(typeid(*n)==typeid(Or))
		{
		assert(false); // Ceci n'est pas une forme disjonctive (les Ors ont d�j� �t� trait�s)
		}
	else if(typeid(*n)==typeid(Expo))
		{
		Expo *e = (Expo*)n;
		val = exploDefinition(e->n1,solution);
		if(!exploDefinition(e->n2,solution))
			val = false;
		if(e->free)
			solution->numericalvariableSolution[e->var->id].seeoutside = true;
		}
	else if(typeid(*n)==typeid(Variable))
		{
		Variable *v = (Variable*)n;
		solution->variableSolution[v->id].seeoutside = true;
		val = true;
		}
	return val;
	}

/**
 *  A function that verifies if a given solution is a real solution,
 *  ie. can be satisfied. We then list the solutions.
 *  If not, THIS is not a solution but there may be another one that is valid.
 */
bool Unification::verifValidPresburger(Node* n)
{
#ifdef DEBUG
    (*mycout) << "------------------------------------------------------------------------------" << endl;
#endif
    if(typeid(*n) == typeid(And)){ // test if there are (is a) Numerical Value Condition
        list<Automata*> automL;
        std::map <int,int> numvars; // trick to count without duplicates
        // OU UN TABLEAU
        list<Condition*>::iterator it = ((And*)n)->conditions.begin();
        while(it != ((And*)n)->conditions.end()) {
            Condition* tmpc = *it;
            if (numvars.find(tmpc->n1->id) == numvars.end()) 
                numvars[tmpc->n1->id] = numvars.size();
            if (typeid(*tmpc) == typeid(CondN)) {
                if (numvars.find(((CondN*)tmpc)->n2->id) == numvars.end())
                    numvars[((CondN*)tmpc)->n2->id] = numvars.size();
            } else if (typeid(*tmpc) == typeid (CondANplusB)) {
                if (numvars.find(((CondANplusB*)tmpc)->n2->id) == numvars.end())
                    numvars[((CondANplusB*)tmpc)->n2->id] = numvars.size();
            } else if (typeid(*tmpc) == typeid (CondNplusN)) {
                if (numvars.find(((CondNplusN*)tmpc)->n2->id) == numvars.end())
                    numvars[((CondNplusN*)tmpc)->n2->id] = numvars.size();
                if (numvars.find(((CondNplusN*)tmpc)->n3->id) == numvars.end())
                    numvars[((CondNplusN*)tmpc)->n3->id] = numvars.size();
            } // only CondN left and it is already taken into account
            it++;
        }
        /* vector<int> numvars; // trick to count without duplicates
        // OU UN TABLEAU
        list<Condition*>::iterator it = ((And*)n)->conditions.begin();
        while(it != ((And*)n)->conditions.end()) {
            Condition* tmpc = *it;
            if (numvars[tmpc->n1->id] == numvars.end()) 
                numvars[tmpc->n1->id] = numvars.size();
            if (typeid(*tmpc) == typeid(CondN)) {
                if (numvars.find(((CondN*)tmpc)->n2->id) == numvars.end())
                    numvars[((CondN*)tmpc)->n2->id] = numvars.size();
            } else if (typeid(*tmpc) == typeid (CondANplusB)) {
                if (numvars.find(((CondANplusB*)tmpc)->n2->id) == numvars.end())
                    numvars[((CondANplusB*)tmpc)->n2->id] = numvars.size();
            } else if (typeid(*tmpc) == typeid (CondNplusN)) {
                if (numvars.find(((CondNplusN*)tmpc)->n2->id) == numvars.end())
                    numvars[((CondNplusN*)tmpc)->n2->id] = numvars.size();
                if (numvars.find(((CondNplusN*)tmpc)->n3->id) == numvars.end())
                    numvars[((CondNplusN*)tmpc)->n3->id] = numvars.size();
            } // only CondN left and it is already taken into account
            it++;
        }
        */

#ifdef DEBUG
        (*mycout) << "==> BITS_SIZE (numvars.size()) : " << numvars.size() << endl;
#endif

        it = ((And*)n)->conditions.begin();
        // build the automatas and push them into a list
        while(it != ((And*)n)->conditions.end()) {
            Automata* tmp = new Automata(*it, &numvars);
#ifdef DEBUG
            tmp->display(*mycout);
#endif
            automL.push_back(tmp);
            it++;
        }

        // now merge the automatas with the help of the bits map ;)

		// builds automates groupes

		list<Groupe> groupes;
		list<Automata*>::iterator it7 = automL.begin();
		while(it7!=automL.end())
			{
			Groupe tmp;
			tmp.autos.push_back(*it7);
			groupes.push_back(tmp);
			it7++;
			}

		list<Groupe>::iterator itg1,itg2;
		list<Automata*>::iterator ita1,ita2;

		bool todo;
		do
			{
			todo = false;
			itg1 = groupes.begin();
			while(itg1!=groupes.end())
				{
				itg2 = itg1;
				if(itg2!=groupes.end())
					itg2++;
				while(itg2!=groupes.end())
					{
					ita1 = itg1->autos.begin();
					while(ita1!=itg1->autos.end())
						{
						ita2 = itg2->autos.begin();
						while(ita2!=itg2->autos.end())
							{
							if((*ita1)->isIntersect(*ita2,numvars.size()))
								{
								todo = true;
								(*itg1).autos.push_back(*ita2);
								(*itg2).autos.erase(ita2);
								if((*itg2).autos.empty())
									groupes.erase(itg2);
								goto groupeend;
								}
							ita2++;
							}
						ita1++;
						}
					itg2++;
					}
				itg1++;
				}
groupeend:;
			}
		while(todo);
#ifdef DEBUG
		(*mycout) << "NDGroupes : " << groupes.size() << endl;
#endif	

		// tests automates groupes
		itg1 = groupes.begin();
		while(itg1!=groupes.end())
			{
			ExploPara exploPara;
			StatePara para;
			bool isgooda = false;

			// Initial state
			list<Automata*>::iterator it2 = itg1->autos.begin();
			while(it2!=itg1->autos.end())
				{
				para.actives.push_back((*it2)->si);
				it2++;
				}
			exploPara.statePara.push_back(para);

			// While empty
			while(!exploPara.statePara.empty())
				{
				para = exploPara.statePara.front();
				exploPara.statePara.pop_front();

				bool allfinal = true;
				list<State*>::iterator it3 = para.actives.begin();
				while(it3!=para.actives.end())
					{
					if((*it3)->value != (*it3)->automata->cvalue)
						{
						allfinal = false;
						break;
						}
					it3++;
					}

				if(allfinal)
					{
					isgooda = true;
					goto gooda;
					}

				State* first = para.actives.front();

				if(para.actives.empty())
					continue;
				if(first->step.empty())
					continue;

				list<Transition*>::iterator it4 = first->step.begin();
				while(it4!=first->step.end())
					{
					StatePara newPara;
					newPara.actives.push_back((*it4)->st);

					bool good2 = true;
					it3 = para.actives.begin();
					it3++;
					while(it3!=para.actives.end())
						{
						bool good = false;
						list<Transition*>::iterator it5 = (*it3)->step.begin();
						while(it5!=(*it3)->step.end())
							{
							// chercher *it5 "=" *it4
							//if(!memcmp((*it5)->bits,(*it4)->bits,sizeof(int)*numvars.size()))
							
							unsigned int i;
							bool err = false;
							for(i=0;i<numvars.size();i++)
							if(((*it5)->bits[i]==-1)||((*it4)->bits[i]==-1)||((*it4)->bits[i]==(*it5)->bits[i]))
								;
							else
								err = true;

							if(!err)
								{
								newPara.actives.push_back((*it5)->st);
								good = true;
								break;
								}

							it5++;
							}

						if(!good)
							{
							good2 = false;
							break;
							}
						it3++;
						}

					if(!good2)
						break;

					exploPara.statePara.push_back(newPara);
					it4++;
					}
				}

gooda:;
			if(!isgooda)
				return false;
			itg1++;
			}

		return true;
    }
    // trivial case : no Numerical Value Condition
    return true;
}

/**
* check all solutions (and delete the bad ones).
*/
bool Unification::checkDefinition()
	{
	if(typeid(*parser->result)==typeid(Or)) // many different solutions that we split
		{
		list<Node*>::iterator it = ((Or*)(parser->result))->branchs.begin();
		while(it!=((Or*)(parser->result))->branchs.end())
			{
			SimpleSolution *sol = new SimpleSolution((unsigned int)parser->variabletab.size(),(unsigned int)parser->numericalvariabletab.size(),this);
			solutions.push_back(sol);
			sol->fail = !exploDefinition(*it,sol);
			if(configuration->presBurgerVerif)
			if(!sol->fail)
				sol->fail = !verifValidPresburger(*it);
			/*
			Solver::Graph *gr = new Solver::Graph(*it,this);
			solutions.push_back(gr->run());
			*/
			it++;
			}
		}
	else // only one solution so there is no need to split
		{
		SimpleSolution *sol = new SimpleSolution((unsigned int)parser->variabletab.size(),(unsigned int)parser->numericalvariabletab.size(),this);
		solutions.push_back(sol);
		sol->fail = !exploDefinition(parser->result,sol);
		if(configuration->presBurgerVerif)
		if(!sol->fail)
			sol->fail = !verifValidPresburger(parser->result);
		/*
		Solver::Graph *gr = new Solver::Graph(parser->result,this);
		solutions.push_back(gr->run());
		*/
		}

	list<Solution*>::iterator it = solutions.begin();
	while(it!=solutions.end())
		{
		if((*it)->check())
			it++;
		else
			it = solutions.erase(it);
		}
	return !solutions.empty();
	}

/**
* display a solution
*/
void SimpleSolution::display(ostream &o)
	{
	unsigned int i;
	for(i=0;i<ndnumvar;i++)
		{
		if(numericalvariableSolution[i].definitionc.empty())
			{
			if((numericalvariableSolution[i].seeoutside)||(!unification->parser->numericalvariabletab[i].automatic))
				{
				o << " " <<unification->parser->numericalvariabletab[i].name << " = *free*";
				o << endl;
				if(globconfig->writeMode==Configuration::LATEX)
					o << endl;
				}
			}
		else
			{
			list<Condition*>::iterator it = numericalvariableSolution[i].definitionc.begin();
			while(it!=numericalvariableSolution[i].definitionc.end())
				{
				o << " ";
				(*it++)->print(o);
				o << endl;
				if(globconfig->writeMode==Configuration::LATEX)
					o << endl;
				}
			}
		}
	for(i=0;i<ndvar;i++)
		{
		o << " " <<unification->parser->variabletab[i].name << " = ";
		if(!variableSolution[i].definition)
			o << "*free*";
		else
			variableSolution[i].definition->print(o);
		o << endl;
		if(globconfig->writeMode==Configuration::LATEX)
			o << endl;
		}
	}

VariableSolution::VariableSolution()
	{
	seeoutside = false;
	definition = NULL;
	}

SimpleSolution::SimpleSolution(unsigned int ndvar,unsigned int ndnumvar,Unification *unification)
	{
	this->unification = unification;
	fail = false;
	this->ndvar = ndvar;
	this->ndnumvar = ndnumvar;
	variableSolution = new VariableSolution[ndvar];
	numericalvariableSolution = new VariableSolution[ndnumvar];
	}

bool SimpleSolution::check()
	{
	if(fail)
		return false;
	return true;
	}

SimpleSolution::~SimpleSolution()
	{
	delete[] variableSolution;
	delete[] numericalvariableSolution;
	}

/**
* Run unification
*/
void Unification::run()
	{
	result = true;
	int step = 0;
	long startTime = clock();

	Display::texDi(*mycout);
	(*mycout) << "Initial set of equation";
	Display::texDi(*mycout);
	Display::texEndl(*mycout);
	(*mycout) << endl;
	debugDisplay((*mycout));
	Display::texEndl(*mycout);
	(*mycout) << endl;

	while(true)
		{
		step++;
		if((configuration->pauseEveryStep)||(configuration->printRule))
			{
			Display::texEndl(*mycout);
			(*mycout) << endl;
			debugDisplay((*mycout));
			//Display::texEndl(*mycout);
			(*mycout) << endl;
			}

		if(configuration->pauseEveryStep)
			{
			(*mycout) << "==============PAUSE=("<< step <<")===========" << endl;
			if(configuration->writeMode==Configuration::LATEX)
				(*mycout) << "\\\\";
			while(true)
				{
				(*mycout) << "command:"<<endl;
				(*mycout) << " c - continue (default)" << endl;
				(*mycout) << " f - finish" << endl;
				(*mycout) << " q - quit" << endl;
				(*mycout) << " i - info" << endl;

				char c = getc(stdin);
				fflush(stdin);

				if((c=='c')||(c==10))
					break;
				else if(c=='f')
					{
					configuration->pauseEveryStep = false;
					break;
					}
				else if(c=='q')
					exit(0);
				else if(c=='i')
					{
					(*mycout) << endl;
					(*mycout) << " Number of steps      : " << step << endl;
					(*mycout) << " Number of terms      : " << Node::ndterms << endl;
					(*mycout) << " Number of variables  : " << parser->variabletab.size() << endl;
					(*mycout) << " Number of functions  : " << parser->fonctiontab.size() << endl;
					(*mycout) << " Number of numericals : " << parser->numericalvariabletab.size() << endl;
					(*mycout) << endl;
					}
				else
					printf("<Unknow command>\n");
				}
			}

		if(configuration->beep!=0)
			Display::beep(Node::ndterms*10,20);

		parser->checkSoundness();

		if(!tryApplyR1())
		if(!tryApplyR2())
		if(!tryApplyR3())
		if(!tryApplyR4())
		if(!tryApplyR5())
			break;

		parser->checkSoundness();

		this->parser->result = ApplyR0();

		// plus des calculs
		//parser->result = SetDisjunctiveSystem();
		}

	this->parser->result = ApplyR0();
	
	if(configuration->disjonction)
		parser->result = SetDisjunctiveSystem();

	Display::texDi(*mycout);
	(*mycout) << "End of computation";
	Display::texDi(*mycout);
	Display::texEndl(*mycout);
	(*mycout) << endl << endl;


	(*mycout) << " Number of steps  : " << step << endl;

	if(globconfig->writeMode==Configuration::LATEX)
		(*mycout) << endl;

	(*mycout) << " Computation time : ";
	long computationtime = (clock() - startTime) *1000 / CLOCKS_PER_SEC; // ms
	Display::displayTime(computationtime);
	Display::texEndl(*mycout);
	(*mycout) << endl;

	if(configuration->printRule)
		{
		(*mycout) << endl;
		debugDisplay((*mycout));
		}

	if(configuration->disjonction)
	if(result)
		result = checkDefinition();
	}

Unification::~Unification()
	{

	}
